@@ -13,13 +13,14 @@ import (
 
 	"github.com/aws/aws-sdk-go/aws"
 	"github.com/hashicorp/go-secure-stdlib/strutil"
-
 	"github.com/hashicorp/vault/sdk/framework"
+	"github.com/hashicorp/vault/sdk/helper/pluginidentityutil"
+	"github.com/hashicorp/vault/sdk/helper/pluginutil"
 	"github.com/hashicorp/vault/sdk/logical"
 )
 
 func (b *backend) pathConfigClient() *framework.Path {
-	return &framework.Path{
+	p := &framework.Path{
 		Pattern: "config/client$",
 
 		DisplayAttrs: &framework.DisplayAttributes{
@@ -86,6 +87,12 @@ func (b *backend) pathConfigClient() *framework.Path {
 				Default:     aws.UseServiceDefaultRetries,
 				Description: "Maximum number of retries for recoverable exceptions of AWS APIs",
 			},
+
+			"role_arn": {
+				Type:        framework.TypeString,
+				Default:     "",
+				Description: "Role ARN to assume for plugin identity token federation",
+			},
 		},
 
 		ExistenceCheck: b.pathConfigClientExistenceCheck,
@@ -122,6 +129,9 @@ func (b *backend) pathConfigClient() *framework.Path {
 		HelpSynopsis:    pathConfigClientHelpSyn,
 		HelpDescription: pathConfigClientHelpDesc,
 	}
+	pluginidentityutil.AddPluginIdentityTokenFields(p.Fields)
+
+	return p
 }
 
 // Establishes dichotomy of request operation between CreateOperation and UpdateOperation.
@@ -169,18 +179,22 @@ func (b *backend) pathConfigClientRead(ctx context.Context, req *logical.Request
 		return nil, nil
 	}
 
+	configData := map[string]interface{}{
+		"access_key":                 clientConfig.AccessKey,
+		"endpoint":                   clientConfig.Endpoint,
+		"iam_endpoint":               clientConfig.IAMEndpoint,
+		"sts_endpoint":               clientConfig.STSEndpoint,
+		"sts_region":                 clientConfig.STSRegion,
+		"use_sts_region_from_client": clientConfig.UseSTSRegionFromClient,
+		"iam_server_id_header_value": clientConfig.IAMServerIdHeaderValue,
+		"max_retries":                clientConfig.MaxRetries,
+		"allowed_sts_header_values":  clientConfig.AllowedSTSHeaderValues,
+		"role_arn":                   clientConfig.RoleARN,
+	}
+
+	clientConfig.PopulatePluginIdentityTokenData(configData)
 	return &logical.Response{
-		Data: map[string]interface{}{
-			"access_key":                 clientConfig.AccessKey,
-			"endpoint":                   clientConfig.Endpoint,
-			"iam_endpoint":               clientConfig.IAMEndpoint,
-			"sts_endpoint":               clientConfig.STSEndpoint,
-			"sts_region":                 clientConfig.STSRegion,
-			"use_sts_region_from_client": clientConfig.UseSTSRegionFromClient,
-			"iam_server_id_header_value": clientConfig.IAMServerIdHeaderValue,
-			"max_retries":                clientConfig.MaxRetries,
-			"allowed_sts_header_values":  clientConfig.AllowedSTSHeaderValues,
-		},
+		Data: configData,
 	}, nil
 }
 
@@ -335,6 +349,41 @@ func (b *backend) pathConfigClientCreateUpdate(ctx context.Context, req *logical
 		configEntry.MaxRetries = data.Get("max_retries").(int)
 	}
 
+	roleArnStr, ok := data.GetOk("role_arn")
+	if ok {
+		if configEntry.RoleARN != roleArnStr.(string) {
+			changedCreds = true
+			configEntry.RoleARN = roleArnStr.(string)
+		}
+	} else if req.Operation == logical.CreateOperation {
+		configEntry.RoleARN = data.Get("role_arn").(string)
+	}
+
+	if err := configEntry.ParsePluginIdentityTokenFields(data); err != nil {
+		return logical.ErrorResponse(err.Error()), nil
+	}
+
+	// handle mutual exclusivity
+	if configEntry.IdentityTokenAudience != "" && configEntry.AccessKey != "" {
+		return logical.ErrorResponse("only one of 'access_key' or 'identity_token_audience' can be set"), nil
+	}
+
+	if configEntry.IdentityTokenAudience != "" && configEntry.RoleARN == "" {
+		return logical.ErrorResponse("role_arn must be set when identity_token_audience is set"), nil
+	}
+
+	if configEntry.IdentityTokenAudience != "" {
+		_, err := b.System().GenerateIdentityToken(ctx, &pluginutil.IdentityTokenRequest{
+			Audience: configEntry.IdentityTokenAudience,
+		})
+		if err != nil {
+			if errors.Is(err, pluginidentityutil.ErrPluginWorkloadIdentityUnsupported) {
+				return logical.ErrorResponse(err.Error()), nil
+			}
+			return nil, err
+		}
+	}
+
 	// Since this endpoint supports both create operation and update operation,
 	// the error checks for access_key and secret_key not being set are not present.
 	// This allows calling this endpoint multiple times to provide the values.
@@ -374,6 +423,8 @@ func (b *backend) configClientToEntry(conf *clientConfig) (*logical.StorageEntry
 // Struct to hold 'aws_access_key' and 'aws_secret_key' that are required to
 // interact with the AWS EC2 API.
 type clientConfig struct {
+	pluginidentityutil.PluginIdentityTokenParams
+
 	AccessKey              string   `json:"access_key"`
 	SecretKey              string   `json:"secret_key"`
 	Endpoint               string   `json:"endpoint"`
@@ -384,6 +435,7 @@ type clientConfig struct {
 	IAMServerIdHeaderValue string   `json:"iam_server_id_header_value"`
 	AllowedSTSHeaderValues []string `json:"allowed_sts_header_values"`
 	MaxRetries             int      `json:"max_retries"`
+	RoleARN                string   `json:"role_arn"`
 }
 
 func (c *clientConfig) validateAllowedSTSHeaderValues(headers http.Header) error {
