@@ -6,8 +6,7 @@ package command
 import (
 	"testing"
 
-	"github.com/mitchellh/cli"
-
+	"github.com/hashicorp/cli"
 	"github.com/hashicorp/vault/command/token"
 )
 
