@@ -8,18 +8,22 @@ import (
 	"flag"
 	"fmt"
 	"io"
-	"io/ioutil"
+	"net/http"
 	"os"
 	"regexp"
 	"strings"
 	"sync"
 	"time"
 
+	"github.com/hashicorp/cli"
+	hcpvlib "github.com/hashicorp/vault-hcp-lib"
 	"github.com/hashicorp/vault/api"
-	"github.com/hashicorp/vault/command/token"
+	"github.com/hashicorp/vault/api/cliconfig"
+	"github.com/hashicorp/vault/api/tokenhelper"
+	"github.com/hashicorp/vault/command/config"
 	"github.com/hashicorp/vault/helper/namespace"
 	"github.com/mattn/go-isatty"
-	"github.com/mitchellh/cli"
+	"github.com/mitchellh/go-homedir"
 	"github.com/pkg/errors"
 	"github.com/posener/complete"
 )
@@ -63,12 +67,14 @@ type BaseCommand struct {
 	flagOutputCurlString bool
 	flagOutputPolicy     bool
 	flagNonInteractive   bool
+	addrWarning          string
 
 	flagMFA []string
 
 	flagHeader map[string]string
 
-	tokenHelper token.TokenHelper
+	tokenHelper    tokenhelper.TokenHelper
+	hcpTokenHelper hcpvlib.HCPTokenHelper
 
 	client *api.Client
 }
@@ -78,6 +84,15 @@ type BaseCommand struct {
 func (c *BaseCommand) Client() (*api.Client, error) {
 	// Read the test client if present
 	if c.client != nil {
+		// Ignoring homedir errors here and moving on to avoid
+		// spamming user with warnings/errors that homedir isn't set.
+		path, err := homedir.Dir()
+		if err == nil {
+			if err := c.applyHCPConfig(path); err != nil {
+				return nil, err
+			}
+		}
+
 		return c.client, nil
 	}
 
@@ -91,7 +106,7 @@ func (c *BaseCommand) Client() (*api.Client, error) {
 		config.Address = c.flagAddress
 	}
 	if c.flagAgentProxyAddress != "" {
-		config.Address = c.flagAgentProxyAddress
+		config.AgentAddress = c.flagAgentProxyAddress
 	}
 
 	if c.flagOutputCurlString {
@@ -186,32 +201,89 @@ func (c *BaseCommand) Client() (*api.Client, error) {
 
 	c.client = client
 
+	// Ignoring homedir errors here and moving on to avoid
+	// spamming user with warnings/errors that homedir isn't set.
+	path, err := homedir.Dir()
+	if err == nil {
+		if err := c.applyHCPConfig(path); err != nil {
+			return nil, err
+		}
+	}
+
+	if c.addrWarning != "" && c.UI != nil {
+		if os.Getenv("VAULT_ADDR") == "" && !c.flags.hadAddressFlag {
+			if !c.flagNonInteractive && isatty.IsTerminal(os.Stdin.Fd()) {
+				c.UI.Warn(wrapAtLength(c.addrWarning))
+			}
+		}
+	}
+
 	return client, nil
 }
 
+func (c *BaseCommand) applyHCPConfig(path string) error {
+	if c.hcpTokenHelper == nil {
+		c.hcpTokenHelper = c.HCPTokenHelper()
+	}
+
+	hcpToken, err := c.hcpTokenHelper.GetHCPToken(path)
+	if err != nil {
+		return err
+	}
+
+	if hcpToken != nil {
+		cookie := &http.Cookie{
+			Name:    "hcp_access_token",
+			Value:   hcpToken.AccessToken,
+			Expires: hcpToken.AccessTokenExpiry,
+		}
+
+		if err := c.client.SetHCPCookie(cookie); err != nil {
+			return fmt.Errorf("unable to correctly connect to the HCP Vault cluster; please reconnect to HCP: %w", err)
+		}
+
+		if err := c.client.SetAddress(hcpToken.ProxyAddr); err != nil {
+			return fmt.Errorf("unable to correctly set the HCP address: %w", err)
+		}
+
+		// remove address warning since address was set to HCP's address
+		c.addrWarning = ""
+	}
+
+	return nil
+}
+
 // SetAddress sets the token helper on the command; useful for the demo server and other outside cases.
 func (c *BaseCommand) SetAddress(addr string) {
 	c.flagAddress = addr
 }
 
 // SetTokenHelper sets the token helper on the command.
-func (c *BaseCommand) SetTokenHelper(th token.TokenHelper) {
+func (c *BaseCommand) SetTokenHelper(th tokenhelper.TokenHelper) {
 	c.tokenHelper = th
 }
 
 // TokenHelper returns the token helper attached to the command.
-func (c *BaseCommand) TokenHelper() (token.TokenHelper, error) {
+func (c *BaseCommand) TokenHelper() (tokenhelper.TokenHelper, error) {
 	if c.tokenHelper != nil {
 		return c.tokenHelper, nil
 	}
 
-	helper, err := DefaultTokenHelper()
+	helper, err := cliconfig.DefaultTokenHelper()
 	if err != nil {
 		return nil, err
 	}
 	return helper, nil
 }
 
+// HCPTokenHelper returns the HCPToken helper attached to the command.
+func (c *BaseCommand) HCPTokenHelper() hcpvlib.HCPTokenHelper {
+	if c.hcpTokenHelper != nil {
+		return c.hcpTokenHelper
+	}
+	return config.DefaultHCPTokenHelper()
+}
+
 // DefaultWrappingLookupFunc is the default wrapping function based on the
 // CLI flag.
 func (c *BaseCommand) DefaultWrappingLookupFunc(operation, path string) string {
@@ -321,10 +393,12 @@ func (c *BaseCommand) flagSet(bit FlagSetBit) *FlagSets {
 				Completion: complete.PredictAnything,
 				Usage:      "Address of the Vault server.",
 			}
+
 			if c.flagAddress != "" {
 				addrStringVar.Default = c.flagAddress
 			} else {
 				addrStringVar.Default = "https://127.0.0.1:8200"
+				c.addrWarning = fmt.Sprintf("WARNING! VAULT_ADDR and -address unset. Defaulting to %s.", addrStringVar.Default)
 			}
 			f.StringVar(addrStringVar)
 
@@ -552,6 +626,10 @@ type FlagSets struct {
 	hiddens     map[string]struct{}
 	completions complete.Flags
 	ui          cli.Ui
+	// hadAddressFlag signals if the FlagSet had an -address
+	// flag set, for the purposes of warning (see also:
+	// BaseCommand::addrWarning).
+	hadAddressFlag bool
 }
 
 // NewFlagSets creates a new flag sets.
@@ -560,7 +638,7 @@ func NewFlagSets(ui cli.Ui) *FlagSets {
 
 	// Errors and usage are controlled by the CLI.
 	mainSet.Usage = func() {}
-	mainSet.SetOutput(ioutil.Discard)
+	mainSet.SetOutput(io.Discard)
 
 	return &FlagSets{
 		flagSets:    make([]*FlagSet, 0, 6),
@@ -594,6 +672,15 @@ type (
 // Parse parses the given flags, returning any errors.
 // Warnings, if any, regarding the arguments format are sent to stdout
 func (f *FlagSets) Parse(args []string, opts ...ParseOptions) error {
+	// Before parsing, check to see if we have an address flag, for the
+	// purposes of warning later. This must be done now, as the argument
+	// will be removed during parsing.
+	for _, arg := range args {
+		if strings.HasPrefix(arg, "-address") {
+			f.hadAddressFlag = true
+		}
+	}
+
 	err := f.mainSet.Parse(args)
 
 	displayFlagWarningsDisabled := false
