@@ -11,10 +11,10 @@ import (
 	"strings"
 
 	"github.com/fatih/structs"
+	"github.com/hashicorp/cli"
 	"github.com/hashicorp/go-secure-stdlib/password"
 	"github.com/hashicorp/vault/api"
 	"github.com/hashicorp/vault/helper/pgpkeys"
-	"github.com/mitchellh/cli"
 	"github.com/posener/complete"
 )
 
