@@ -1,4 +1,4 @@
-#!/bin/env bash
+#!/usr/bin/env bash
 # Copyright (c) HashiCorp, Inc.
 # SPDX-License-Identifier: BUSL-1.1
 
@@ -9,18 +9,36 @@ fail() {
   exit 1
 }
 
+[[ -z "$IP_VERSION" ]] && fail "IP_VERSION env variable has not been set"
+[[ -z "$NETCAT_COMMAND" ]] && fail "NETCAT_COMMAND env variable has not been set"
 [[ -z "$SOCKET_PORT" ]] && fail "SOCKET_PORT env variable has not been set"
 
+if [ "$IP_VERSION" = "4" ]; then
+  export SOCKET_ADDR="127.0.0.1"
+else
+  export SOCKET_ADDR="::1"
+fi
+
 socket_listener_procs() {
-  pgrep -x nc
+  pgrep -x "${NETCAT_COMMAND}"
 }
 
 kill_socket_listener() {
-  pkill nc
+  pkill  "${NETCAT_COMMAND}"
 }
 
 test_socket_listener() {
-  nc -zvw 2 127.0.0.1 "$SOCKET_PORT" < /dev/null
+  case $IP_VERSION in
+    4)
+      "${NETCAT_COMMAND}" -zvw 2 "${SOCKET_ADDR}" "$SOCKET_PORT" < /dev/null
+      ;;
+    6)
+      "${NETCAT_COMMAND}" -6 -zvw 2 "${SOCKET_ADDR}" "$SOCKET_PORT" < /dev/null
+      ;;
+    *)
+      fail "unknown IP_VERSION: $IP_VERSION"
+      ;;
+  esac
 }
 
 start_socket_listener() {
@@ -32,7 +50,17 @@ start_socket_listener() {
   # Run nc to listen on port 9090 for the socket auditor. We spawn nc
   # with nohup to ensure that the listener doesn't expect a SIGHUP and
   # thus block the SSH session from exiting or terminating on exit.
-  nohup nc -kl "$SOCKET_PORT" >> /tmp/vault-socket.log 2>&1 < /dev/null &
+  case $IP_VERSION in
+    4)
+      nohup nc -kl "$SOCKET_PORT" >> /tmp/vault-socket.log 2>&1 < /dev/null &
+      ;;
+    6)
+      nohup nc -6 -kl "$SOCKET_PORT" >> /tmp/vault-socket.log 2>&1 < /dev/null &
+      ;;
+    *)
+      fail "unknown IP_VERSION: $IP_VERSION"
+      ;;
+  esac
 }
 
 read_log() {
