@@ -162,6 +162,7 @@ func TestSysConfigState_Sanitized(t *testing.T) {
 				"max_lease_ttl":                       json.Number("0"),
 				"pid_file":                            "",
 				"plugin_directory":                    "",
+				"plugin_tmpdir":                       "",
 				"plugin_file_uid":                     json.Number("0"),
 				"plugin_file_permissions":             json.Number("0"),
 				"enable_response_header_hostname":     false,
@@ -176,6 +177,8 @@ func TestSysConfigState_Sanitized(t *testing.T) {
 				"storage":                       tc.expectedStorageOutput,
 				"administrative_namespace_path": "",
 				"imprecise_lease_role_tracking": false,
+				"enable_post_unseal_trace":      false,
+				"post_unseal_trace_directory":   "",
 			}
 
 			if tc.expectedHAStorageOutput != nil {
@@ -190,6 +193,7 @@ func TestSysConfigState_Sanitized(t *testing.T) {
 				"warnings":       nil,
 				"auth":           nil,
 				"data":           configResp,
+				"mount_type":     "system",
 			}
 
 			testResponseBody(t, resp, &actual)
