@@ -15,6 +15,7 @@ import (
 	"github.com/hashicorp/vault/helper/testhelpers/corehelpers"
 	"github.com/hashicorp/vault/helper/versions"
 	"github.com/hashicorp/vault/sdk/helper/consts"
+	"github.com/hashicorp/vault/sdk/logical"
 	"github.com/hashicorp/vault/vault"
 )
 
@@ -34,6 +35,7 @@ func TestSysMounts(t *testing.T) {
 		"wrap_info":      nil,
 		"warnings":       nil,
 		"auth":           nil,
+		"mount_type":     "system",
 		"data": map[string]interface{}{
 			"secret/": map[string]interface{}{
 				"description":             "key/value secret storage",
@@ -215,6 +217,7 @@ func TestSysMount(t *testing.T) {
 		"wrap_info":      nil,
 		"warnings":       nil,
 		"auth":           nil,
+		"mount_type":     "system",
 		"data": map[string]interface{}{
 			"foo/": map[string]interface{}{
 				"description":             "foo",
@@ -529,6 +532,7 @@ func TestSysRemount(t *testing.T) {
 		"wrap_info":      nil,
 		"warnings":       nil,
 		"auth":           nil,
+		"mount_type":     "system",
 		"data": map[string]interface{}{
 			"bar/": map[string]interface{}{
 				"description":             "foo",
@@ -742,6 +746,7 @@ func TestSysUnmount(t *testing.T) {
 		"wrap_info":      nil,
 		"warnings":       nil,
 		"auth":           nil,
+		"mount_type":     "system",
 		"data": map[string]interface{}{
 			"secret/": map[string]interface{}{
 				"description":             "key/value secret storage",
@@ -930,6 +935,7 @@ func TestSysTuneMount_Options(t *testing.T) {
 		"wrap_info":      nil,
 		"warnings":       nil,
 		"auth":           nil,
+		"mount_type":     "system",
 		"data": map[string]interface{}{
 			"description":       "foo",
 			"default_lease_ttl": json.Number("2764800"),
@@ -968,6 +974,7 @@ func TestSysTuneMount_Options(t *testing.T) {
 		"wrap_info":      nil,
 		"warnings":       nil,
 		"auth":           nil,
+		"mount_type":     "system",
 		"data": map[string]interface{}{
 			"description":       "foo",
 			"default_lease_ttl": json.Number("2764800"),
@@ -988,7 +995,12 @@ func TestSysTuneMount_Options(t *testing.T) {
 }
 
 func TestSysTuneMount(t *testing.T) {
-	core, _, token := vault.TestCoreUnsealed(t)
+	coreConfig := &vault.CoreConfig{
+		LogicalBackends: map[string]logical.Factory{
+			"kv": vault.LeasedPassthroughBackendFactory,
+		},
+	}
+	core, _, token := vault.TestCoreUnsealedWithConfig(t, coreConfig)
 	ln, addr := TestServer(t, core)
 	defer ln.Close()
 	TestServerAuth(t, addr, token)
@@ -1009,6 +1021,7 @@ func TestSysTuneMount(t *testing.T) {
 		"wrap_info":      nil,
 		"warnings":       nil,
 		"auth":           nil,
+		"mount_type":     "system",
 		"data": map[string]interface{}{
 			"foo/": map[string]interface{}{
 				"description":             "foo",
@@ -1248,6 +1261,7 @@ func TestSysTuneMount(t *testing.T) {
 		"wrap_info":      nil,
 		"warnings":       nil,
 		"auth":           nil,
+		"mount_type":     "system",
 		"data": map[string]interface{}{
 			"foo/": map[string]interface{}{
 				"description":             "foo",
@@ -1446,6 +1460,7 @@ func TestSysTuneMount(t *testing.T) {
 		"wrap_info":      nil,
 		"warnings":       nil,
 		"auth":           nil,
+		"mount_type":     "system",
 		"data": map[string]interface{}{
 			"description":       "foo",
 			"default_lease_ttl": json.Number("259196400"),
@@ -1484,6 +1499,7 @@ func TestSysTuneMount(t *testing.T) {
 		"wrap_info":      nil,
 		"warnings":       nil,
 		"auth":           nil,
+		"mount_type":     "system",
 		"data": map[string]interface{}{
 			"description":       "foobar",
 			"default_lease_ttl": json.Number("40"),
@@ -1581,6 +1597,7 @@ func TestSysTuneMount_nonHMACKeys(t *testing.T) {
 		"wrap_info":      nil,
 		"warnings":       nil,
 		"auth":           nil,
+		"mount_type":     "system",
 		"data": map[string]interface{}{
 			"description":                  "key/value secret storage",
 			"default_lease_ttl":            json.Number("2764800"),
@@ -1627,6 +1644,7 @@ func TestSysTuneMount_nonHMACKeys(t *testing.T) {
 		"wrap_info":      nil,
 		"warnings":       nil,
 		"auth":           nil,
+		"mount_type":     "system",
 		"data": map[string]interface{}{
 			"description":       "key/value secret storage",
 			"default_lease_ttl": json.Number("2764800"),
@@ -1665,6 +1683,7 @@ func TestSysTuneMount_listingVisibility(t *testing.T) {
 		"wrap_info":      nil,
 		"warnings":       nil,
 		"auth":           nil,
+		"mount_type":     "system",
 		"data": map[string]interface{}{
 			"description":       "key/value secret storage",
 			"default_lease_ttl": json.Number("2764800"),
@@ -1702,6 +1721,7 @@ func TestSysTuneMount_listingVisibility(t *testing.T) {
 		"wrap_info":      nil,
 		"warnings":       nil,
 		"auth":           nil,
+		"mount_type":     "system",
 		"data": map[string]interface{}{
 			"description":        "key/value secret storage",
 			"default_lease_ttl":  json.Number("2764800"),
@@ -1748,6 +1768,7 @@ func TestSysTuneMount_passthroughRequestHeaders(t *testing.T) {
 		"wrap_info":      nil,
 		"warnings":       nil,
 		"auth":           nil,
+		"mount_type":     "system",
 		"data": map[string]interface{}{
 			"description":                 "key/value secret storage",
 			"default_lease_ttl":           json.Number("2764800"),
@@ -1787,6 +1808,7 @@ func TestSysTuneMount_passthroughRequestHeaders(t *testing.T) {
 		"wrap_info":      nil,
 		"warnings":       nil,
 		"auth":           nil,
+		"mount_type":     "system",
 		"data": map[string]interface{}{
 			"description":       "key/value secret storage",
 			"default_lease_ttl": json.Number("2764800"),
@@ -1831,6 +1853,7 @@ func TestSysTuneMount_allowedManagedKeys(t *testing.T) {
 		"wrap_info":      nil,
 		"warnings":       nil,
 		"auth":           nil,
+		"mount_type":     "system",
 		"data": map[string]interface{}{
 			"description":          "key/value secret storage",
 			"default_lease_ttl":    json.Number("2764800"),
@@ -1870,6 +1893,7 @@ func TestSysTuneMount_allowedManagedKeys(t *testing.T) {
 		"wrap_info":      nil,
 		"warnings":       nil,
 		"auth":           nil,
+		"mount_type":     "system",
 		"data": map[string]interface{}{
 			"description":       "key/value secret storage",
 			"default_lease_ttl": json.Number("2764800"),
