@@ -122,9 +122,17 @@ func ParseConfig(d string) (*SharedConfig, error) {
 
 	if o := list.Filter("listener"); len(o.Items) > 0 {
 		result.found("listener", "Listener")
-		if err := ParseListeners(&result, o); err != nil {
+		listeners, err := ParseListeners(o)
+		if err != nil {
 			return nil, fmt.Errorf("error parsing 'listener': %w", err)
 		}
+		// Update the shared config
+		result.Listeners = listeners
+
+		// Track which types of listener were found.
+		for _, l := range result.Listeners {
+			result.found(l.Type.String(), l.Type.String())
+		}
 	}
 
 	if o := list.Filter("user_lockout"); len(o.Items) > 0 {
