@@ -1,4 +1,7 @@
-// +build !enterprise
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
+//go:build !enterprise
 
 package configutil
 
@@ -6,8 +9,7 @@ import (
 	"github.com/hashicorp/hcl/hcl/ast"
 )
 
-type EntSharedConfig struct {
-}
+type EntSharedConfig struct{}
 
 func (ec *EntSharedConfig) ParseConfig(list *ast.ObjectList) error {
 	return nil
