@@ -22,7 +22,7 @@ func TestPostgreSQLBackend(t *testing.T) {
 	// Use docker as pg backend if no url is provided via environment variables
 	connURL := os.Getenv("PGURL")
 	if connURL == "" {
-		cleanup, u := postgresql.PrepareTestContainer(t, "11.1")
+		cleanup, u := postgresql.PrepareTestContainer(t)
 		defer cleanup()
 		connURL = u
 	}
@@ -159,7 +159,7 @@ func TestConnectionURL(t *testing.T) {
 	for name, tt := range cases {
 		t.Run(name, func(t *testing.T) {
 			// This is necessary to avoid always testing the branch where the env is set.
-			// As long the the env is set --- even if the value is "" --- `ok` returns true.
+			// As long the env is set --- even if the value is "" --- `ok` returns true.
 			if tt.input.envar != "" {
 				os.Setenv("VAULT_PG_CONNECTION_URL", tt.input.envar)
 				defer os.Unsetenv("VAULT_PG_CONNECTION_URL")
