@@ -5,10 +5,9 @@
 
 import Component from '@ember/component';
 import { task } from 'ember-concurrency';
-import { getOwner } from '@ember/application';
-import { inject as service } from '@ember/service';
+import { getOwner } from '@ember/owner';
+import { service } from '@ember/service';
 import { alias } from '@ember/object/computed';
-import { AbortController } from 'fetch';
 
 export default Component.extend({
   file: null,
