@@ -4,7 +4,7 @@
  */
 
 import Component from '@glimmer/component';
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 import { inject as controller } from '@ember/controller';
 
 export default class SidebarNavComponent extends Component {
