@@ -4,10 +4,11 @@
  */
 
 import Component from '@glimmer/component';
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 
 export default class SidebarNavClusterComponent extends Component {
   @service currentCluster;
+  @service flags;
   @service version;
   @service auth;
   @service namespace;
@@ -16,8 +17,12 @@ export default class SidebarNavClusterComponent extends Component {
     return this.currentCluster.cluster;
   }
 
+  get hasChrootNamespace() {
+    return this.cluster?.hasChrootNamespace;
+  }
+
   get isRootNamespace() {
     // should only return true if we're in the true root namespace
-    return this.namespace.inRootNamespace && !this.currentCluster.hasChrootNamespace;
+    return this.namespace.inRootNamespace && !this.hasChrootNamespace;
   }
 }
