@@ -4,7 +4,7 @@
  */
 
 import Component from '@glimmer/component';
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 import { later } from '@ember/runloop';
 import { action } from '@ember/object';
 import { tracked } from '@glimmer/tracking';
@@ -21,6 +21,9 @@ export default class SidebarUserMenuComponent extends Component {
     // in order to use the MFA end user setup they need an entity_id
     return !!this.auth.authData?.entity_id;
   }
+  get isUserpass() {
+    return this.auth.authData?.backend?.type === 'userpass';
+  }
 
   get isRenewing() {
     return this.fakeRenew || this.auth.isRenewing;
