@@ -3,16 +3,28 @@
  * SPDX-License-Identifier: BUSL-1.1
  */
 
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 import Controller from '@ember/controller';
 import ListController from 'core/mixins/list-controller';
 
 export default Controller.extend(ListController, {
   flashMessages: service(),
 
+  entityToDisable: null,
+  itemToDelete: null,
+
+  // callback from HDS pagination to set the queryParams page
+  get paginationQueryParams() {
+    return (page) => {
+      return {
+        page,
+      };
+    };
+  },
+
   actions: {
     delete(model) {
-      const type = model.get('identityType');
+      const type = model.identityType;
       const id = model.id;
       return model
         .destroyRecord()
@@ -24,12 +36,13 @@ export default Controller.extend(ListController, {
           this.flashMessages.success(
             `There was a problem deleting ${type}: ${id} - ${e.errors.join(' ') || e.message}`
           );
-        });
+        })
+        .finally(() => this.set('itemToDelete', null));
     },
 
     toggleDisabled(model) {
-      const action = model.get('disabled') ? ['enabled', 'enabling'] : ['disabled', 'disabling'];
-      const type = model.get('identityType');
+      const action = model.disabled ? ['enabled', 'enabling'] : ['disabled', 'disabling'];
+      const type = model.identityType;
       const id = model.id;
       model.toggleProperty('disabled');
 
@@ -42,7 +55,8 @@ export default Controller.extend(ListController, {
           this.flashMessages.success(
             `There was a problem ${action[1]} ${type}: ${id} - ${e.errors.join(' ') || e.message}`
           );
-        });
+        })
+        .finally(() => this.set('entityToDisable', null));
     },
     reloadRecord(model) {
       model.reload();
