@@ -5,7 +5,7 @@
 
 import Controller from '@ember/controller';
 import { action } from '@ember/object';
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 
 export default class OidcAssignmentDetailsController extends Controller {
   @service router;
