@@ -4,11 +4,14 @@
  */
 
 import Controller from '@ember/controller';
+import { service } from '@ember/service';
 
 export default Controller.extend({
+  router: service(),
+
   actions: {
     onMountSuccess: function (type, path) {
-      const transition = this.transitionToRoute('vault.cluster.settings.auth.configure', path);
+      const transition = this.router.transitionTo('vault.cluster.settings.auth.configure', path);
       return transition.followRedirects();
     },
   },
