@@ -3,11 +3,14 @@
  * SPDX-License-Identifier: BUSL-1.1
  */
 
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 import Controller from '@ember/controller';
 
 export default Controller.extend({
   auth: service(),
+  router: service(),
+  version: service(),
+  store: service(),
 
   actions: {
     seal() {
@@ -15,9 +18,11 @@ export default Controller.extend({
         .adapterFor('cluster')
         .seal()
         .then(() => {
-          this.model.cluster.get('leaderNode').set('sealed', true);
+          this.store.peekAll('cluster')[0].reload();
           this.auth.deleteCurrentToken();
-          return this.transitionToRoute('vault.cluster.unseal');
+          // Reset version so it doesn't show on footer
+          this.version.version = null;
+          return this.router.transitionTo('vault.cluster.unseal');
         });
     },
   },
