@@ -4,7 +4,7 @@
  */
 
 import Model, { attr, belongsTo, hasMany } from '@ember-data/model';
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 import { get } from '@ember/object';
 
 export default class ClusterModel extends Model {
@@ -16,8 +16,10 @@ export default class ClusterModel extends Model {
   @attr('boolean') standby;
   @attr('string') type;
   @attr('object') license;
-  // manually set on response when sys/health failure
+
+  // manually set on response in cluster adapter
   @attr('boolean') hasChrootNamespace;
+  @attr('boolean') replicationRedacted;
 
   /* Licensing concerns */
   get licenseExpiry() {
@@ -76,7 +78,9 @@ export default class ClusterModel extends Model {
 
   //replication mode - will only ever be 'unsupported'
   //otherwise the particular mode will have the relevant mode attr through replication-attributes
-  @attr('string') mode;
+  // eg dr.mode or performance.mode
+  @attr('string')
+  mode;
   get allReplicationDisabled() {
     return this.dr?.replicationDisabled && this.performance?.replicationDisabled;
   }
