@@ -14,7 +14,7 @@ export default IdentityModel.extend({
   formFields: computed(function () {
     return ['name', 'mountAccessor'];
   }),
-  entity: belongsTo('identity/entity', { readOnly: true, async: false }),
+  entity: belongsTo('identity/entity', { readOnly: true, async: false, inverse: 'aliases' }),
 
   name: attr('string'),
   canonicalId: attr('string'),
@@ -24,6 +24,7 @@ export default IdentityModel.extend({
   }),
   metadata: attr({
     editType: 'kv',
+    isSectionHeader: true,
   }),
   mountPath: attr('string', {
     readOnly: true,
