@@ -6,7 +6,7 @@
 import Model, { belongsTo, attr } from '@ember-data/model';
 
 export default Model.extend({
-  config: belongsTo('kmip/config', { async: false }),
+  config: belongsTo('kmip/config', { async: false, inverse: 'ca' }),
   caPem: attr('string', {
     label: 'CA PEM',
   }),
