@@ -1,18 +1,21 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: BUSL-1.1
+ */
+
+import { attr } from '@ember-data/model';
 import { computed } from '@ember/object';
-import DS from 'ember-data';
 
 import PolicyModel from '../policy';
 import { expandAttributeMeta } from 'vault/utils/field-to-attrs';
 
-let { attr } = DS;
-
 export default PolicyModel.extend({
   enforcementLevel: attr('string', {
     possibleValues: ['advisory', 'soft-mandatory', 'hard-mandatory'],
     defaultValue: 'hard-mandatory',
   }),
 
-  additionalAttrs: computed(function() {
+  additionalAttrs: computed(function () {
     return expandAttributeMeta(this, ['enforcementLevel']);
   }),
 });
