@@ -5,15 +5,16 @@
 
 import Route from '@ember/routing/route';
 import UnloadModelRoute from 'vault/mixins/unload-model-route';
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 
 export default Route.extend(UnloadModelRoute, {
   store: service(),
+  router: service(),
 
   beforeModel() {
     const itemType = this.modelFor('vault.cluster.access.identity');
     if (itemType !== 'entity') {
-      return this.transitionTo('vault.cluster.access.identity');
+      return this.router.transitionTo('vault.cluster.access.identity');
     }
   },
 
