@@ -4,9 +4,10 @@
  */
 
 import Route from '@ember/routing/route';
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 
 export default class MfaConfigureRoute extends Route {
+  @service router;
   @service store;
 
   beforeModel() {
@@ -14,7 +15,7 @@ export default class MfaConfigureRoute extends Route {
       .query('mfa-method', {})
       .then(() => {
         // if response then they should transition to the methods page instead of staying on the configure page.
-        this.transitionTo('vault.cluster.access.mfa.methods.index');
+        this.router.transitionTo('vault.cluster.access.mfa.methods.index');
       })
       .catch(() => {
         // stay on the landing page
