@@ -1,8 +1,13 @@
-import { decamelize } from '@ember/string';
-import DS from 'ember-data';
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: BUSL-1.1
+ */
 
-export default DS.RESTSerializer.extend({
-  keyForAttribute: function(attr) {
+import RESTSerializer from '@ember-data/serializer/rest';
+import { decamelize } from '@ember/string';
+
+export default RESTSerializer.extend({
+  keyForAttribute: function (attr) {
     return decamelize(attr);
   },
 
