@@ -3,7 +3,8 @@
  * SPDX-License-Identifier: BUSL-1.1
  */
 
-import Service, { inject as service } from '@ember/service';
+import Service, { service } from '@ember/service';
+import { tracked } from '@glimmer/tracking';
 import { sanitizePath, sanitizeStart } from 'core/utils/sanitize-path';
 import { task } from 'ember-concurrency';
 
@@ -45,6 +46,15 @@ const API_PATHS = {
     activity: 'sys/internal/counters/activity',
     config: 'sys/internal/counters/config',
   },
+  settings: {
+    customMessages: 'sys/config/ui/custom-messages',
+  },
+  sync: {
+    destinations: 'sys/sync/destinations',
+    associations: 'sys/sync/associations',
+    config: 'sys/sync/config',
+    github: 'sys/sync/github-apps',
+  },
 };
 
 const API_PATHS_TO_ROUTE_PARAMS = {
@@ -63,26 +73,46 @@ const API_PATHS_TO_ROUTE_PARAMS = {
   It fetches a users' policy from the resultant-acl endpoint and stores their
   allowed exact and glob paths as state. It also has methods for checking whether
   a user has permission for a given path.
+  The data from the resultant-acl endpoint has the following shape:
+  {
+    exact_paths: {
+      [key: string]: {
+        capabilities: string[];
+      };
+    };
+    glob_paths: {
+      [key: string]: {
+        capabilities: string[];
+      };
+    };
+    root: boolean;
+    chroot_namespace?: string;
+  };
+  There are a couple nuances to be aware of about this response. When a
+  chroot_namespace is set, all of the paths in the response will be prefixed
+  with that namespace. Additionally, this endpoint is only added to the default
+  policy in the user's root namespace, so we make the call to the user's root
+  namespace (the namespace where the user's auth method is mounted) no matter
+  what the current namespace is.
 */
 
-export default Service.extend({
-  exactPaths: null,
-  globPaths: null,
-  canViewAll: null,
-  permissionsBanner: null,
-  chrootNamespace: null,
-  store: service(),
-  auth: service(),
-  namespace: service(),
+export default class PermissionsService extends Service {
+  @tracked exactPaths = null;
+  @tracked globPaths = null;
+  @tracked canViewAll = null;
+  @tracked permissionsBanner = null;
+  @tracked chrootNamespace = null;
+  @service store;
+  @service namespace;
 
-  get baseNs() {
+  get fullCurrentNamespace() {
     const currentNs = this.namespace.path;
     return this.chrootNamespace
       ? `${sanitizePath(this.chrootNamespace)}/${sanitizePath(currentNs)}`
       : sanitizePath(currentNs);
-  },
+  }
 
-  getPaths: task(function* () {
+  @task *getPaths() {
     if (this.paths) {
       return;
     }
@@ -93,38 +123,76 @@ export default Service.extend({
       return;
     } catch (err) {
       // If no policy can be found, default to showing all nav items.
-      this.set('canViewAll', true);
-      this.set('permissionsBanner', PERMISSIONS_BANNER_STATES.readFailed);
+      this.canViewAll = true;
+      this.permissionsBanner = PERMISSIONS_BANNER_STATES.readFailed;
     }
-  }),
+  }
 
+  /**
+   * hasWildcardNsAccess checks if the user has a wildcard access to target namespace
+   * via full glob path or any ancestors of the target namespace
+   * @param {string} targetNs is the current/target namespace that we are checking access for
+   * @param {object} globPaths key is path, value is object with capabilities
+   * @returns {boolean} whether the user's policy includes wildcard access to NS
+   */
+  hasWildcardNsAccess(targetNs, globPaths = {}) {
+    const nsParts = sanitizePath(targetNs).split('/');
+    let matchKey = null;
+    // For each section of the namespace, check if there is a matching wildcard path
+    while (nsParts.length > 0) {
+      // glob paths always end in a slash
+      const test = `${nsParts.join('/')}/`;
+      if (Object.keys(globPaths).includes(test)) {
+        matchKey = test;
+        break;
+      }
+      nsParts.pop();
+    }
+    // Finally, check if user has wildcard access to the root namespace
+    // which is represented by an empty string
+    if (!matchKey && Object.keys(globPaths).includes('')) {
+      matchKey = '';
+    }
+    if (null === matchKey) {
+      return false;
+    }
+
+    // if there is a match make sure the capabilities do not include deny
+    return !this.isDenied(globPaths[matchKey]);
+  }
+
+  // This method is called to recalculate whether to show the permissionsBanner when the namespace changes
   calcNsAccess() {
     if (this.canViewAll) {
-      this.set('permissionsBanner', null);
+      this.permissionsBanner = null;
       return;
     }
-    const namespace = this.baseNs;
+    const namespace = this.fullCurrentNamespace;
     const allowed =
-      Object.keys(this.globPaths).any((k) => k.startsWith(namespace)) ||
-      Object.keys(this.exactPaths).any((k) => k.startsWith(namespace));
-    this.set('permissionsBanner', allowed ? null : PERMISSIONS_BANNER_STATES.noAccess);
-  },
+      // check if the user has wildcard access to the relative root namespace
+      this.hasWildcardNsAccess(namespace, this.globPaths) ||
+      // or if any of their glob paths start with the namespace
+      Object.keys(this.globPaths).any((k) => k.startsWith(namespace) && !this.isDenied(this.globPaths[k])) ||
+      // or if any of their exact paths start with the namespace
+      Object.keys(this.exactPaths).any((k) => k.startsWith(namespace) && !this.isDenied(this.exactPaths[k]));
+    this.permissionsBanner = allowed ? null : PERMISSIONS_BANNER_STATES.noAccess;
+  }
 
   setPaths(resp) {
-    this.set('exactPaths', resp.data.exact_paths);
-    this.set('globPaths', resp.data.glob_paths);
-    this.set('canViewAll', resp.data.root);
-    this.set('chrootNamespace', resp.data.chroot_namespace);
+    this.exactPaths = resp.data.exact_paths;
+    this.globPaths = resp.data.glob_paths;
+    this.canViewAll = resp.data.root;
+    this.chrootNamespace = resp.data.chroot_namespace;
     this.calcNsAccess();
-  },
+  }
 
   reset() {
-    this.set('exactPaths', null);
-    this.set('globPaths', null);
-    this.set('canViewAll', null);
-    this.set('chrootNamespace', null);
-    this.set('permissionsBanner', null);
-  },
+    this.exactPaths = null;
+    this.globPaths = null;
+    this.canViewAll = null;
+    this.chrootNamespace = null;
+    this.permissionsBanner = null;
+  }
 
   hasNavPermission(navItem, routeParams, requireAll) {
     if (routeParams) {
@@ -139,7 +207,7 @@ export default Service.extend({
       });
     }
     return Object.values(API_PATHS[navItem]).some((path) => this.hasPermission(path));
-  },
+  }
 
   navPathParams(navItem) {
     const path = Object.values(API_PATHS[navItem]).find((path) => this.hasPermission(path));
@@ -148,16 +216,16 @@ export default Service.extend({
     }
 
     return API_PATHS_TO_ROUTE_PARAMS[path];
-  },
+  }
 
   pathNameWithNamespace(pathName) {
-    const namespace = this.baseNs;
+    const namespace = this.fullCurrentNamespace;
     if (namespace) {
       return `${sanitizePath(namespace)}/${sanitizeStart(pathName)}`;
     } else {
       return pathName;
     }
-  },
+  }
 
   hasPermission(pathName, capabilities = [null]) {
     if (this.canViewAll) {
@@ -169,7 +237,7 @@ export default Service.extend({
       (capability) =>
         this.hasMatchingExactPath(path, capability) || this.hasMatchingGlobPath(path, capability)
     );
-  },
+  }
 
   hasMatchingExactPath(pathName, capability) {
     const exactPaths = this.exactPaths;
@@ -184,7 +252,7 @@ export default Service.extend({
       return hasMatchingPath;
     }
     return false;
-  },
+  }
 
   hasMatchingGlobPath(pathName, capability) {
     const globPaths = this.globPaths;
@@ -203,13 +271,13 @@ export default Service.extend({
       return hasMatchingPath;
     }
     return false;
-  },
+  }
 
   hasCapability(path, capability) {
     return path.capabilities.includes(capability);
-  },
+  }
 
   isDenied(path) {
     return path.capabilities.includes('deny');
-  },
-});
+  }
+}
