@@ -10,25 +10,13 @@ import { debounce, next } from '@ember/runloop';
 import type { HTMLElementEvent } from 'vault/forms';
 
 interface Args {
-  value?: string; // initial value
-  placeholder?: string; // defaults to Type to filter results
-  wait?: number; // defaults to 200
+  wait?: number; // defaults to 500
   autofocus?: boolean; // initially focus the input on did-insert
-  onInput(value: string): void;
+  hideIcon?: boolean; // hide the search icon in the input
+  onInput(value: string): void; // invoked with input value after debounce timer expires
 }
 
 export default class FilterInputComponent extends Component<Args> {
-  value: string | undefined;
-
-  constructor(owner: unknown, args: Args) {
-    super(owner, args);
-    this.value = this.args.value;
-  }
-
-  get placeholder() {
-    return this.args.placeholder || 'Type to filter results';
-  }
-
   @action
   focus(elem: HTMLElement) {
     if (this.args.autofocus) {
@@ -38,11 +26,10 @@ export default class FilterInputComponent extends Component<Args> {
 
   @action
   onInput(event: HTMLElementEvent<HTMLInputElement>) {
-    const callback = () => {
-      this.args.onInput(event.target.value);
-    };
-    const wait = this.args.wait || 200;
+    const wait = this.args.wait || 500;
     // ts complains when trying to pass object of optional args to callback as 3rd arg to debounce
-    debounce(this, callback, wait);
+    // eslint-disable-next-line
+    // @ts-ignore
+    debounce(this, this.args.onInput, event.target.value, wait);
   }
 }
