@@ -7,13 +7,12 @@ import { alias } from '@ember/object/computed';
 import Component from '@ember/component';
 import ReplicationActions from 'core/mixins/replication-actions';
 import layout from '../templates/components/replication-actions';
-import { A } from '@ember/array';
 
 const DEFAULTS = {
   token: null,
   primary_api_addr: null,
   primary_cluster_addr: null,
-  errors: A(),
+  errors: null,
   id: null,
   force: false,
 };
