@@ -5,7 +5,7 @@
 
 import { withConfirmLeave } from 'core/decorators/confirm-leave';
 import Route from '@ember/routing/route';
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 
 @withConfirmLeave()
 export default class PkiKeyEditRoute extends Route {
@@ -18,9 +18,9 @@ export default class PkiKeyEditRoute extends Route {
   setupController(controller, resolvedModel) {
     super.setupController(controller, resolvedModel);
     controller.breadcrumbs = [
-      { label: 'secrets', route: 'secrets', linkExternal: true },
-      { label: this.secretMountPath.currentPath, route: 'overview' },
-      { label: 'keys', route: 'keys.index' },
+      { label: 'Secrets', route: 'secrets', linkExternal: true },
+      { label: this.secretMountPath.currentPath, route: 'overview', model: this.secretMountPath.currentPath },
+      { label: 'keys', route: 'keys.index', model: this.secretMountPath.currentPath },
       { label: resolvedModel.id },
     ];
   }
