@@ -4,7 +4,7 @@
  */
 
 import Route from '@ember/routing/route';
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 import { withConfirmLeave } from 'core/decorators/confirm-leave';
 
 withConfirmLeave();
@@ -23,10 +23,10 @@ export default class PkiRoleGenerateRoute extends Route {
     super.setupController(controller, resolvedModel);
     const { role } = this.paramsFor('roles/role');
     controller.breadcrumbs = [
-      { label: 'secrets', route: 'secrets', linkExternal: true },
-      { label: this.secretMountPath.currentPath, route: 'overview' },
-      { label: 'roles', route: 'roles.index' },
-      { label: role, route: 'roles.role.details' },
+      { label: 'Secrets', route: 'secrets', linkExternal: true },
+      { label: this.secretMountPath.currentPath, route: 'overview', model: this.secretMountPath.currentPath },
+      { label: 'roles', route: 'roles.index', model: this.secretMountPath.currentPath },
+      { label: role, route: 'roles.role.details', models: [this.secretMountPath.currentPath, role] },
       { label: 'generate certificate' },
     ];
     // This is updated on successful generate in the controller
