@@ -4,16 +4,17 @@
  */
 
 import { alias } from '@ember/object/computed';
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 import Controller from '@ember/controller';
 
 export default Controller.extend({
   flashMessages: service(),
+  router: service(),
   rm: service('replication-mode'),
   replicationMode: alias('rm.mode'),
   actions: {
     resetConfig(config) {
-      if (config.get('isNew')) {
+      if (config.isNew) {
         config.setProperties({
           mode: null,
           paths: [],
@@ -29,14 +30,15 @@ export default Controller.extend({
       const flash = this.flashMessages;
       const id = config.id;
       const redirectArgs = isDelete
-        ? ['mode.secondaries', this.replicationMode]
-        : ['mode.secondaries.config-show', id];
+        ? ['vault.cluster.replication.mode.secondaries', this.replicationMode]
+        : ['vault.cluster.replication.mode.secondaries.config-show', id];
       const modelMethod = isDelete ? config.destroyRecord : config.save;
 
       modelMethod
         .call(config)
         .then(() => {
-          this.transitionToRoute(...redirectArgs)
+          this.router
+            .transitionTo(...redirectArgs)
             .followRedirects()
             .then(() => {
               flash.success(
