@@ -12,8 +12,8 @@ export default Base.extend({
 
   redirect(model) {
     const replicationMode = this.paramsFor('mode').replication_mode;
-    if (!model.get(`${replicationMode}.isPrimary`) || !model.get('canAddSecondary')) {
-      return this.transitionTo('mode', replicationMode);
+    if (!model[replicationMode].isPrimary || !model.canAddSecondary) {
+      return this.router.transitionTo('vault.cluster.replication.mode', replicationMode);
     }
   },
 
