@@ -3,7 +3,7 @@
  * SPDX-License-Identifier: BUSL-1.1
  */
 
-import { Factory } from 'ember-cli-mirage';
+import { Factory } from 'miragejs';
 
 // For the purposes of testing, we only use a subset of fields relevant to mysql
 export default Factory.extend({
