@@ -3,7 +3,7 @@
  * SPDX-License-Identifier: BUSL-1.1
  */
 
-import { Factory } from 'ember-cli-mirage';
+import { Factory } from 'miragejs';
 /* eslint-disable ember/avoid-leaking-state-in-ember-objects */
 export default Factory.extend({
   openapi: '3.0.2',
