@@ -8,15 +8,13 @@ import { v4 as uuidv4 } from 'uuid';
 import { setupApplicationTest } from 'vault/tests/helpers';
 import authPage from 'vault/tests/pages/auth';
 import { deleteEngineCmd, mountEngineCmd, runCmd, tokenWithPolicyCmd } from 'vault/tests/helpers/commands';
-import { personas } from 'vault/tests/helpers/policy-generator/kv';
-import {
-  clearRecords,
-  deleteLatestCmd,
-  setupControlGroup,
-  writeVersionedSecret,
-} from 'vault/tests/helpers/kv/kv-run-commands';
-import { click, currentURL, visit } from '@ember/test-helpers';
+import { personas } from 'vault/tests/helpers/kv/policy-generator';
+import { clearRecords, deleteLatestCmd, writeVersionedSecret } from 'vault/tests/helpers/kv/kv-run-commands';
+import { setupControlGroup } from 'vault/tests/helpers/control-groups';
+import { click, currentRouteName, currentURL, waitUntil, visit } from '@ember/test-helpers';
 import { PAGE } from 'vault/tests/helpers/kv/kv-selectors';
+import sinon from 'sinon';
+import { GENERAL } from 'vault/tests/helpers/general-selectors';
 
 const ALL_DELETE_ACTIONS = ['delete', 'destroy', 'undelete'];
 const assertDeleteActions = (assert, expected = ['delete', 'destroy']) => {
@@ -28,6 +26,11 @@ const assertDeleteActions = (assert, expected = ['delete', 'destroy']) => {
     }
   });
 };
+
+const makeToken = (name, mountPath, policyGenerator) => {
+  return tokenWithPolicyCmd(`${name}-${mountPath}`, policyGenerator(mountPath));
+};
+
 /**
  * This test set is for testing delete, undelete, destroy flows
  * Letter(s) in parenthesis at the end are shorthand for the persona,
@@ -45,9 +48,7 @@ module('Acceptance | kv-v2 workflow | delete, undelete, destroy', function (hook
     await runCmd(mountEngineCmd('kv-v2', this.backend), false);
     await writeVersionedSecret(this.backend, this.secretPath, 'foo', 'bar', 4);
     await writeVersionedSecret(this.backend, this.nestedSecretPath, 'foo', 'bar', 1);
-    await writeVersionedSecret(this.backend, 'nuke', 'foo', 'bar', 2);
-    // Delete latest version for testing undelete for users that can't delete
-    await runCmd(deleteLatestCmd(this.backend, 'nuke'));
+    // Versioned secret for testing delete is created (and deleted) by each module to avoid race condition failures
     return;
   });
 
@@ -58,18 +59,23 @@ module('Acceptance | kv-v2 workflow | delete, undelete, destroy', function (hook
 
   module('admin persona', function (hooks) {
     hooks.beforeEach(async function () {
-      const token = await runCmd(tokenWithPolicyCmd('admin', personas.admin(this.backend)));
+      // patch is an enterprise feature but stubbing the version so assertions that check
+      // patch actions exist before/after deletion can run on both CE and ent repos
+      this.version = this.owner.lookup('service:version').type = 'enterprise';
+      const token = await runCmd(makeToken('admin', this.backend, personas.admin));
       await authPage.login(token);
       clearRecords(this.store);
       return;
     });
     test('can delete and undelete the latest secret version (a)', async function (assert) {
-      assert.expect(17);
+      assert.expect(21);
+      const flashSuccess = sinon.spy(this.owner.lookup('service:flash-messages'), 'success');
       // go to secret details
       await visit(`/vault/secrets/${this.backend}/kv/${this.secretPath}/details`);
       // correct toolbar options & details show
       assertDeleteActions(assert);
-      assert.dom(PAGE.infoRow).exists('shows secret data');
+      assert.dom(PAGE.detail.patchLatest).exists();
+      assert.dom(PAGE.infoRow).exists('shows secret data on load');
       // delete flow
       await click(PAGE.detail.delete);
       assert.dom(PAGE.detail.deleteModalTitle).includesText('Delete version?', 'shows correct modal title');
@@ -77,28 +83,37 @@ module('Acceptance | kv-v2 workflow | delete, undelete, destroy', function (hook
       assert.dom(PAGE.detail.deleteOptionLatest).isNotDisabled('delete latest option is selectable');
       await click(PAGE.detail.deleteOptionLatest);
       await click(PAGE.detail.deleteConfirm);
+      const expected = `Successfully deleted Version 4 of ${this.secretPath}.`;
+      const [actual] = flashSuccess.lastCall.args;
+      assert.strictEqual(actual, expected, 'renders correct flash message');
+
       // details update accordingly
+      await click(PAGE.secretTab('Secret'));
       assert
         .dom(PAGE.emptyStateTitle)
         .hasText('Version 4 of this secret has been deleted', 'Shows deleted message');
       assert.dom(PAGE.detail.versionTimestamp).includesText('Version 4 deleted');
       // updated toolbar options
       assertDeleteActions(assert, ['undelete', 'destroy']);
+      assert.dom(PAGE.detail.patchLatest).doesNotExist('patching a deleted secret is not allowed');
       // undelete flow
       await click(PAGE.detail.undelete);
       // details update accordingly
-      assert.dom(PAGE.infoRow).exists('shows secret data');
+      await click(PAGE.secretTab('Secret'));
+      assert.dom(PAGE.infoRow).exists('shows secret data after undeleting');
       assert.dom(PAGE.detail.versionTimestamp).includesText('Version 4 created');
       // correct toolbar options
       assertDeleteActions(assert, ['delete', 'destroy']);
+      assert.dom(PAGE.detail.patchLatest).exists('patch is allowed after undeleting');
     });
     test('can soft delete and undelete an older secret version (a)', async function (assert) {
-      assert.expect(17);
+      assert.expect(19);
       // go to secret details
       await visit(`/vault/secrets/${this.backend}/kv/${this.secretPath}/details?version=2`);
       // correct toolbar options & details show
       assertDeleteActions(assert);
-      assert.dom(PAGE.infoRow).exists('shows secret data');
+      assert.dom(PAGE.detail.patchLatest).exists();
+      assert.dom(PAGE.infoRow).exists('shows secret data on load');
       // delete flow
       await click(PAGE.detail.delete);
       assert.dom(PAGE.detail.deleteModalTitle).includesText('Delete version?', 'shows correct modal title');
@@ -106,44 +121,58 @@ module('Acceptance | kv-v2 workflow | delete, undelete, destroy', function (hook
       assert.dom(PAGE.detail.deleteOptionLatest).isNotDisabled('delete latest option is selectable');
       await click(PAGE.detail.deleteOption);
       await click(PAGE.detail.deleteConfirm);
-      // details update accordingly
+      // we get navigated back to the overview page, so manually go back to deleted version
+      await visit(`/vault/secrets/${this.backend}/kv/${this.secretPath}/details?version=2`);
       assert
         .dom(PAGE.emptyStateTitle)
         .hasText('Version 2 of this secret has been deleted', 'Shows deleted message');
       assert.dom(PAGE.detail.versionTimestamp).includesText('Version 2 deleted');
       // updated toolbar options
       assertDeleteActions(assert, ['undelete', 'destroy']);
+      assert
+        .dom(PAGE.detail.patchLatest)
+        .exists('patching the latest version is allowed after deleting an older version');
+
       // undelete flow
       await click(PAGE.detail.undelete);
       // details update accordingly
-      assert.dom(PAGE.infoRow).exists('shows secret data');
+      await visit(`/vault/secrets/${this.backend}/kv/${this.secretPath}/details?version=2`);
+      assert.dom(PAGE.infoRow).exists('shows secret data after undeleting');
       assert.dom(PAGE.detail.versionTimestamp).includesText('Version 2 created');
       // correct toolbar options
       assertDeleteActions(assert, ['delete', 'destroy']);
     });
     test('can destroy a secret version (a)', async function (assert) {
-      assert.expect(9);
+      assert.expect(12);
+      const flashSuccess = sinon.spy(this.owner.lookup('service:flash-messages'), 'success');
       // go to secret details
       await visit(`/vault/secrets/${this.backend}/kv/${this.secretPath}/details?version=3`);
       // correct toolbar options show
       assertDeleteActions(assert);
+      assert.dom(PAGE.detail.patchLatest).exists();
       // delete flow
       await click(PAGE.detail.destroy);
       assert.dom(PAGE.detail.deleteModalTitle).includesText('Destroy version?', 'modal has correct title');
       await click(PAGE.detail.deleteConfirm);
+      const expected = `Successfully destroyed Version 3 of ${this.secretPath}.`;
+      const [actual] = flashSuccess.lastCall.args;
+      assert.strictEqual(actual, expected, 'renders correct flash message');
       // details update accordingly
+      await visit(`/vault/secrets/${this.backend}/kv/${this.secretPath}/details?version=3`);
       assert
         .dom(PAGE.emptyStateTitle)
         .hasText('Version 3 of this secret has been permanently destroyed', 'Shows destroyed message');
       assert.dom(PAGE.detail.versionTimestamp).doesNotExist('does not show version timestamp');
       // updated toolbar options
       assertDeleteActions(assert, []);
+      assert
+        .dom(PAGE.detail.patchLatest)
+        .exists('patching the latest version is allowed after destroying an older version');
     });
+
     test('can permanently delete all secret versions (a)', async function (assert) {
-      // go to secret details
-      await visit(`/vault/secrets/${this.backend}/kv/nuke/details`);
-      // Check metadata toolbar
-      await click(PAGE.secretTab('Metadata'));
+      await writeVersionedSecret(this.backend, 'nuke', 'foo', 'bar', 2);
+      await visit(`/vault/secrets/${this.backend}/kv/nuke/metadata`);
       assert.dom(PAGE.metadata.deleteMetadata).hasText('Permanently delete', 'shows delete metadata button');
       // delete flow
       await click(PAGE.metadata.deleteMetadata);
@@ -151,7 +180,7 @@ module('Acceptance | kv-v2 workflow | delete, undelete, destroy', function (hook
         .dom(PAGE.detail.deleteModalTitle)
         .includesText('Delete metadata and secret data?', 'modal has correct title');
       await click(PAGE.detail.deleteConfirm);
-
+      await waitUntil(() => currentRouteName() === 'vault.cluster.secrets.backend.kv.list');
       // redirects to list
       assert.strictEqual(currentURL(), `/vault/secrets/${this.backend}/kv/list`, 'redirects to list');
     });
@@ -159,7 +188,12 @@ module('Acceptance | kv-v2 workflow | delete, undelete, destroy', function (hook
 
   module('data-reader persona', function (hooks) {
     hooks.beforeEach(async function () {
-      const token = await runCmd(tokenWithPolicyCmd('data-reader', personas.dataReader(this.backend)));
+      // create and delete a secret as root user
+      await authPage.login();
+      await writeVersionedSecret(this.backend, 'nuke', 'foo', 'bar', 2);
+      await runCmd(deleteLatestCmd(this.backend, 'nuke'));
+      // login as data-reader persona
+      const token = await runCmd(makeToken('data-reader', this.backend, personas.dataReader));
       await authPage.login(token);
       clearRecords(this.store);
       return;
@@ -206,14 +240,17 @@ module('Acceptance | kv-v2 workflow | delete, undelete, destroy', function (hook
 
   module('data-list-reader persona', function (hooks) {
     hooks.beforeEach(async function () {
-      const token = await runCmd(
-        tokenWithPolicyCmd('data-list-reader', personas.dataListReader(this.backend))
-      );
+      // create and delete a secret as root user
+      await authPage.login();
+      await writeVersionedSecret(this.backend, 'nuke', 'foo', 'bar', 2);
+      await runCmd(deleteLatestCmd(this.backend, 'nuke'));
+      // login as data-list-reader persona
+      const token = await runCmd(makeToken('data-list-reader', this.backend, personas.dataListReader));
       await authPage.login(token);
       clearRecords(this.store);
       return;
     });
-    test('can delete and undelete the latest secret version (dlr)', async function (assert) {
+    test('can delete and cannot undelete the latest secret version (dlr)', async function (assert) {
       assert.expect(12);
       // go to secret details
       await visit(`/vault/secrets/${this.backend}/kv/${this.secretPath}/details`);
@@ -228,6 +265,7 @@ module('Acceptance | kv-v2 workflow | delete, undelete, destroy', function (hook
       await click(PAGE.detail.deleteOptionLatest);
       await click(PAGE.detail.deleteConfirm);
       // details update accordingly
+      await click(PAGE.secretTab('Secret'));
       assert
         .dom(PAGE.emptyStateTitle)
         .hasText('Version 4 of this secret has been deleted', 'Shows deleted message');
@@ -255,7 +293,7 @@ module('Acceptance | kv-v2 workflow | delete, undelete, destroy', function (hook
       // correct toolbar options show
       assertDeleteActions(assert, ['delete']);
     });
-    test('cannot permanently delete all secret versions (dr)', async function (assert) {
+    test('cannot permanently delete all secret versions (dlr)', async function (assert) {
       // go to secret details
       await visit(`/vault/secrets/${this.backend}/kv/nuke/details`);
       // Check metadata toolbar
@@ -266,15 +304,18 @@ module('Acceptance | kv-v2 workflow | delete, undelete, destroy', function (hook
 
   module('metadata-maintainer persona', function (hooks) {
     hooks.beforeEach(async function () {
-      const token = await runCmd(
-        tokenWithPolicyCmd('metadata-maintainer', personas.metadataMaintainer(this.backend))
-      );
+      // create and delete a secret as root user
+      await authPage.login();
+      await writeVersionedSecret(this.backend, 'nuke', 'foo', 'bar', 2);
+      await runCmd(deleteLatestCmd(this.backend, 'nuke'));
+      // login as metadata-maintainer persona
+      const token = await runCmd(makeToken('metadata-maintainer', this.backend, personas.metadataMaintainer));
       await authPage.login(token);
       clearRecords(this.store);
       return;
     });
-    test('can delete and undelete the latest secret version (mm)', async function (assert) {
-      assert.expect(17);
+    test('cannot delete but can undelete the latest secret version (mm)', async function (assert) {
+      assert.expect(18);
       // go to secret details
       await visit(`/vault/secrets/${this.backend}/kv/${this.secretPath}/details`);
       // correct toolbar options & details show
@@ -294,7 +335,12 @@ module('Acceptance | kv-v2 workflow | delete, undelete, destroy', function (hook
       assertDeleteActions(assert, ['undelete', 'destroy']);
       // undelete flow
       await click(PAGE.detail.undelete);
+      await waitUntil(() => currentRouteName() === 'vault.cluster.secrets.backend.kv.secret.index');
+      assert
+        .dom(GENERAL.overviewCard.container('Current version'))
+        .hasText(`Current version The current version of this secret. 2`);
       // details update accordingly
+      await click(PAGE.secretTab('Secret'));
       assert.dom(PAGE.emptyStateTitle).hasText('You do not have permission to read this secret');
       assert.dom(PAGE.detail.versionTimestamp).doesNotExist('Version 2 timestamp not rendered');
       // correct toolbar options
@@ -316,6 +362,7 @@ module('Acceptance | kv-v2 workflow | delete, undelete, destroy', function (hook
       await click(PAGE.detail.deleteOption);
       await click(PAGE.detail.deleteConfirm);
       // details update accordingly
+      await visit(`/vault/secrets/${this.backend}/kv/${this.secretPath}/details?version=2`);
       assert.dom(PAGE.emptyStateTitle).hasText('You do not have permission to read this secret');
       assert.dom(PAGE.detail.versionTimestamp).doesNotExist('Version 2 timestamp not rendered');
       // updated toolbar options
@@ -323,6 +370,7 @@ module('Acceptance | kv-v2 workflow | delete, undelete, destroy', function (hook
       // undelete flow
       await click(PAGE.detail.undelete);
       // details update accordingly
+      await visit(`/vault/secrets/${this.backend}/kv/${this.secretPath}/details?version=2`);
       assert.dom(PAGE.emptyStateTitle).hasText('You do not have permission to read this secret');
       assert.dom(PAGE.detail.versionTimestamp).doesNotExist('Version 2 timestamp not rendered');
       // correct toolbar options
@@ -339,6 +387,7 @@ module('Acceptance | kv-v2 workflow | delete, undelete, destroy', function (hook
       assert.dom(PAGE.detail.deleteModalTitle).includesText('Destroy version?', 'modal has correct title');
       await click(PAGE.detail.deleteConfirm);
       // details update accordingly
+      await visit(`/vault/secrets/${this.backend}/kv/${this.secretPath}/details?version=3`);
       assert
         .dom(PAGE.emptyStateTitle)
         .hasText('You do not have permission to read this secret', 'Shows permissions message');
@@ -358,7 +407,7 @@ module('Acceptance | kv-v2 workflow | delete, undelete, destroy', function (hook
   module('secret-nested-creator persona', function (hooks) {
     hooks.beforeEach(async function () {
       const token = await runCmd(
-        tokenWithPolicyCmd('secret-nested-creator', personas.secretNestedCreator(this.backend))
+        makeToken('secret-nested-creator', this.backend, personas.secretNestedCreator)
       );
       await authPage.login(token);
       clearRecords(this.store);
@@ -384,7 +433,7 @@ module('Acceptance | kv-v2 workflow | delete, undelete, destroy', function (hook
 
   module('secret-creator persona', function (hooks) {
     hooks.beforeEach(async function () {
-      const token = await runCmd(tokenWithPolicyCmd('secret-creator', personas.secretCreator(this.backend)));
+      const token = await runCmd(makeToken('secret-creator', this.backend, personas.secretCreator));
       await authPage.login(token);
       clearRecords(this.store);
       return;
@@ -420,6 +469,7 @@ module('Acceptance | kv-v2 workflow | delete, undelete, destroy', function (hook
       assertDeleteActions(assert, []);
     });
     test('can permanently delete all secret versions (sc)', async function (assert) {
+      await writeVersionedSecret(this.backend, 'nuke', 'foo', 'bar', 2);
       // go to secret details
       await visit(`/vault/secrets/${this.backend}/kv/nuke/details`);
       // Check metadata toolbar
@@ -431,7 +481,7 @@ module('Acceptance | kv-v2 workflow | delete, undelete, destroy', function (hook
         .dom(PAGE.detail.deleteModalTitle)
         .includesText('Delete metadata and secret data?', 'modal has correct title');
       await click(PAGE.detail.deleteConfirm);
-
+      await waitUntil(() => currentRouteName() === 'vault.cluster.secrets.backend.kv.list');
       // redirects to list
       assert.strictEqual(currentURL(), `/vault/secrets/${this.backend}/kv/list`, 'redirects to list');
     });
@@ -468,7 +518,7 @@ path "sys/control-group/request" {
 }
 `;
 
-      const { userToken } = await setupControlGroup({ userPolicy });
+      const { userToken } = await setupControlGroup({ userPolicy, backend: this.backend });
       this.userToken = userToken;
       await authPage.login(userToken);
       clearRecords(this.store);
