@@ -4,7 +4,6 @@
  */
 
 import { click, visit, settled, currentURL, currentRouteName, fillIn } from '@ember/test-helpers';
-import { create } from 'ember-cli-page-object';
 import { module, test } from 'qunit';
 import { setupApplicationTest } from 'ember-qunit';
 import { v4 as uuidv4 } from 'uuid';
@@ -16,17 +15,17 @@ import listPage from 'vault/tests/pages/secrets/backend/list';
 import mountSecrets from 'vault/tests/pages/settings/mount-secret-backend';
 import authPage from 'vault/tests/pages/auth';
 import logout from 'vault/tests/pages/logout';
-import consoleClass from 'vault/tests/pages/components/console/ui-panel';
 import { writeSecret, writeVersionedSecret } from 'vault/tests/helpers/kv/kv-run-commands';
+import { runCmd } from 'vault/tests/helpers/commands';
 import { PAGE } from 'vault/tests/helpers/kv/kv-selectors';
-
-const consoleComponent = create(consoleClass);
+import codemirror from 'vault/tests/helpers/codemirror';
+import { GENERAL } from 'vault/tests/helpers/general-selectors';
 
 const deleteEngine = async function (enginePath, assert) {
   await logout.visit();
   await authPage.login();
-  await consoleComponent.runCommands([`delete sys/mounts/${enginePath}`]);
-  const response = consoleComponent.lastLogOutput;
+
+  const response = await runCmd([`delete sys/mounts/${enginePath}`]);
   assert.strictEqual(
     response,
     `Success! Data deleted (if it existed) at: sys/mounts/${enginePath}`,
@@ -51,8 +50,6 @@ module('Acceptance | secrets/secret/create, read, delete', function (hooks) {
       await mountSecrets.visit();
       await click('[data-test-mount-type="kv"]');
 
-      await click('[data-test-mount-next]');
-
       await fillIn('[data-test-input="path"]', enginePath);
       await fillIn('[data-test-input="maxVersions"]', maxVersion);
       await click('[data-test-input="casRequired"]');
@@ -72,19 +69,19 @@ module('Acceptance | secrets/secret/create, read, delete', function (hooks) {
       assert
         .dom(PAGE.infoRowValue('Automate secret deletion'))
         .hasText('1 second', 'displays the delete version after set when configuring the secret-engine');
+      // [BANDAID] avoid error from missing param for links in SecretEdit > KeyValueHeader
+      await visit('/vault/secrets');
       await deleteEngine(enginePath, assert);
     });
 
     // https://github.com/hashicorp/vault/issues/5994
     test('v1 key named keys', async function (assert) {
       assert.expect(2);
-      await consoleComponent.runCommands([
-        'vault write sys/mounts/test type=kv',
-        'refresh',
-        'vault write test/a keys=a keys=b',
-      ]);
+      await runCmd(['vault write sys/mounts/test type=kv', 'refresh', 'vault write test/a keys=a keys=b']);
       await showPage.visit({ backend: 'test', id: 'a' });
       assert.ok(showPage.editIsPresent, 'renders the page properly');
+      // [BANDAID] avoid error from missing param for links in SecretEdit > KeyValueHeader
+      await visit('/vault/secrets');
       await deleteEngine('test', assert);
     });
   });
@@ -92,16 +89,16 @@ module('Acceptance | secrets/secret/create, read, delete', function (hooks) {
   module('kv v2', function (hooks) {
     hooks.beforeEach(async function () {
       this.backend = `kvv2-${this.uid}`;
-      await consoleComponent.runCommands([`write sys/mounts/${this.backend} type=kv options=version=2`]);
+      await runCmd([`write sys/mounts/${this.backend} type=kv options=version=2`]);
     });
     hooks.afterEach(async function () {
-      await consoleComponent.runCommands([`delete sys/mounts/${this.backend}`]);
+      await runCmd([`delete sys/mounts/${this.backend}`]);
     });
     test('it can create a secret when check-and-set is required', async function (assert) {
       const secretPath = 'foo/bar';
-      await consoleComponent.runCommands(`write ${this.backend}/config cas_required=true`);
+      const output = await runCmd(`write ${this.backend}/config cas_required=true`);
       assert.strictEqual(
-        consoleComponent.lastLogOutput,
+        output,
         `Success! Data written to: ${this.backend}/config`,
         'Engine successfully updated'
       );
@@ -109,15 +106,15 @@ module('Acceptance | secrets/secret/create, read, delete', function (hooks) {
       await writeSecret(this.backend, secretPath, 'foo', 'bar');
       assert.strictEqual(
         currentRouteName(),
-        'vault.cluster.secrets.backend.kv.secret.details.index',
-        'redirects to the show page'
+        'vault.cluster.secrets.backend.kv.secret.index',
+        'redirects to the overview page'
       );
-      assert.dom(PAGE.detail.createNewVersion).exists('shows the edit button');
     });
     test('it navigates to version history and to a specific version', async function (assert) {
       assert.expect(4);
       const secretPath = `specific-version`;
       await writeVersionedSecret(this.backend, secretPath, 'foo', 'bar', 4);
+      await click(PAGE.secretTab('Secret'));
       assert
         .dom(PAGE.detail.versionTimestamp)
         .includesText('Version 4 created', 'shows version created time');
@@ -141,12 +138,13 @@ module('Acceptance | secrets/secret/create, read, delete', function (hooks) {
       // mount version 1 engine
       await mountSecrets.visit();
       await mountSecrets.selectType('kv');
-      await mountSecrets.next().path(this.backend).toggleOptions().version(1).submit();
+      await mountSecrets.path(this.backend).toggleOptions().version(1).submit();
     });
     hooks.afterEach(async function () {
-      await consoleComponent.runCommands([`delete sys/mounts/${this.backend}`]);
+      await runCmd([`delete sys/mounts/${this.backend}`]);
     });
-    test('version 1 performs the correct capabilities lookup and does not show metadata tab', async function (assert) {
+    test('version 1 performs the correct capabilities lookup', async function (assert) {
+      // TODO: while this should pass it doesn't really do anything anymore for us as v1 and v2 are completely separate.
       const secretPath = 'foo/bar';
       await listPage.create();
       await editPage.createSecret(secretPath, 'foo', 'bar');
@@ -156,37 +154,32 @@ module('Acceptance | secrets/secret/create, read, delete', function (hooks) {
         'redirects to the show page'
       );
       assert.ok(showPage.editIsPresent, 'shows the edit button');
-      // check for metadata tab should not exist on KV version 1
-      assert.dom('[data-test-secret-metadata-tab]').doesNotExist('does not show metadata tab');
     });
     // https://github.com/hashicorp/vault/issues/5960
     test('version 1: nested paths creation maintains ability to navigate the tree', async function (assert) {
       const enginePath = this.backend;
-      const secretPath = '1/2/3/4';
-      await listPage.create();
-      await editPage.createSecret(secretPath, 'foo', 'bar');
+      await runCmd([
+        `write ${enginePath}/1/2 foo=bar`,
+        `write ${enginePath}/1/2/3/4 foo=bar`,
+        `write ${enginePath}/1/2/3/4a foo=bar`,
+        'refresh',
+      ]);
+      await settled();
+      // navigate to farthest leaf
+      await visit(`/vault/secrets/${enginePath}/list`);
+      assert.dom('[data-test-component="navigate-input"]').hasNoValue();
+      assert.dom('[data-test-secret-link]').exists({ count: 1 });
+      await click('[data-test-secret-link="1/"]');
+      assert.dom('[data-test-component="navigate-input"]').hasValue('1/');
+      assert.dom('[data-test-secret-link]').exists({ count: 2 });
+      await click('[data-test-secret-link="1/2/"]');
+      assert.dom('[data-test-component="navigate-input"]').hasValue('1/2/');
+      assert.dom('[data-test-secret-link]').exists({ count: 1 });
+      await click('[data-test-secret-link="1/2/3/"]');
+      assert.dom('[data-test-component="navigate-input"]').hasValue('1/2/3/');
+      assert.dom('[data-test-secret-link]').exists({ count: 2 });
 
-      // setup an ancestor for when we delete
-      await listPage.visitRoot({ backend: enginePath });
-      await listPage.secrets.filterBy('text', '1/')[0].click();
-      await listPage.create();
-      await editPage.createSecret('1/2', 'foo', 'bar');
-
-      // lol we have to do this because ember-cli-page-object doesn't like *'s in visitable
-      await listPage.visitRoot({ backend: enginePath });
-      await listPage.secrets.filterBy('text', '1/')[0].click();
-      await listPage.secrets.filterBy('text', '2/')[0].click();
-      await listPage.secrets.filterBy('text', '3/')[0].click();
-      await listPage.create();
-
-      await editPage.createSecret(secretPath + 'a', 'foo', 'bar');
-      await listPage.visitRoot({ backend: enginePath });
-      await listPage.secrets.filterBy('text', '1/')[0].click();
-      await listPage.secrets.filterBy('text', '2/')[0].click();
-      const secretLink = listPage.secrets.filterBy('text', '3/')[0];
-      assert.ok(secretLink, 'link to the 3/ branch displays properly');
-
-      await listPage.secrets.filterBy('text', '3/')[0].click();
+      // delete the items
       await listPage.secrets.objectAt(0).menuToggle();
       await settled();
       await listPage.delete();
@@ -195,17 +188,20 @@ module('Acceptance | secrets/secret/create, read, delete', function (hooks) {
       assert.strictEqual(currentRouteName(), 'vault.cluster.secrets.backend.list');
       assert.strictEqual(currentURL(), `/vault/secrets/${enginePath}/list/1/2/3/`, 'remains on the page');
 
+      assert.dom('[data-test-secret-link]').exists({ count: 1 });
       await listPage.secrets.objectAt(0).menuToggle();
       await listPage.delete();
       await listPage.confirmDelete();
       await settled();
-      assert.strictEqual(currentRouteName(), 'vault.cluster.secrets.backend.list');
-      assert.strictEqual(
-        currentURL(),
-        `/vault/secrets/${enginePath}/list/1/`,
-        'navigates to the ancestor created earlier'
-      );
+      assert.strictEqual(currentURL(), `/vault/secrets/${enginePath}/list/1/2/3/`, 'remains on the page');
+      assert.dom(GENERAL.emptyStateTitle).hasText('No secrets under "1/2/3/".');
+      await fillIn('[data-test-component="navigate-input"]', '1/2/');
+      assert.dom(GENERAL.emptyStateTitle).hasText('No secrets under "1/2/".');
+      await click('[data-test-list-root-link]');
+      assert.strictEqual(currentURL(), `/vault/secrets/${enginePath}/list`);
+      assert.dom('[data-test-secret-link]').exists({ count: 1 });
     });
+
     test('first level secrets redirect properly upon deletion', async function (assert) {
       const secretPath = 'test';
       await listPage.create();
@@ -244,7 +240,7 @@ module('Acceptance | secrets/secret/create, read, delete', function (hooks) {
       assert.expect(paths.length * 2 + 1);
       const secretPath = '2';
       const commands = paths.map((path) => `write '${backend}/${path}/${secretPath}' 3=4`);
-      await consoleComponent.runCommands([...commands, 'refresh']);
+      await runCmd([...commands, 'refresh']);
       for (const path of paths) {
         await listPage.visit({ backend, id: path });
         assert.ok(listPage.secrets.filterBy('text', '2')[0], `${path}: secret is displayed properly`);
@@ -255,16 +251,19 @@ module('Acceptance | secrets/secret/create, read, delete', function (hooks) {
           `${path}: show page renders correctly`
         );
       }
+      // [BANDAID] avoid error from missing param for links in SecretEdit > KeyValueHeader
+      await visit('/vault/secrets');
       await deleteEngine(backend, assert);
     });
 
-    test('UI handles secret with % in path correctly', async function (assert) {
+    test('KVv1 handles secret with % in path correctly', async function (assert) {
       const enginePath = this.backend;
       const secretPath = 'per%cent/%fu ll';
       const [firstPath, secondPath] = secretPath.split('/');
       const commands = [`write '${enginePath}/${secretPath}' 3=4`, `refresh`];
-      await consoleComponent.runCommands(commands);
+      await runCmd(commands);
       await listPage.visitRoot({ backend: enginePath });
+      await settled();
 
       assert.dom(`[data-test-secret-link="${firstPath}/"]`).exists('First section item exists');
       await click(`[data-test-secret-link="${firstPath}/"]`);
@@ -284,7 +283,7 @@ module('Acceptance | secrets/secret/create, read, delete', function (hooks) {
         'secret path is encoded in URL'
       );
       assert.dom('h1').hasText(secretPath, 'Path renders correctly on show page');
-      await click(`[data-test-secret-breadcrumb="${firstPath}"]`);
+      await click(`[data-test-secret-breadcrumb="${firstPath}"] a`);
       assert.strictEqual(
         currentURL(),
         `/vault/secrets/${enginePath}/list/${encodeURIComponent(firstPath)}/`,
@@ -296,7 +295,6 @@ module('Acceptance | secrets/secret/create, read, delete', function (hooks) {
     test('creating a secret with a single or double quote works properly', async function (assert) {
       assert.expect(6);
       const backend = this.backend;
-      // await consoleComponent.runCommands(`write sys/mounts/${backend} type=kv`);
       const paths = ["'some", '"some'];
       for (const path of paths) {
         await listPage.visitRoot({ backend });
@@ -316,7 +314,7 @@ module('Acceptance | secrets/secret/create, read, delete', function (hooks) {
 
     test('filter clears on nav', async function (assert) {
       const backend = this.backend;
-      await consoleComponent.runCommands([
+      await runCmd([
         `vault write sys/mounts/${backend} type=kv`,
         `refresh`,
         `vault write ${backend}/filter/foo keys=a keys=b`,
@@ -328,7 +326,7 @@ module('Acceptance | secrets/secret/create, read, delete', function (hooks) {
       await listPage.filterInput('filter/foo1');
       assert.strictEqual(listPage.secrets.length, 1, 'renders only one secret');
       await listPage.secrets.objectAt(0).click();
-      await showPage.breadcrumbs.filterBy('text', 'filter')[0].click();
+      await click('[data-test-secret-breadcrumb="filter"] a');
       assert.strictEqual(listPage.secrets.length, 3, 'renders three secrets');
       assert.strictEqual(listPage.filterInputValue, 'filter/', 'pageFilter has been reset');
     });
@@ -339,8 +337,7 @@ module('Acceptance | secrets/secret/create, read, delete', function (hooks) {
       await listPage.visitRoot({ backend: this.backend });
       await listPage.create();
       await editPage.path(secretPath).toggleJSON();
-      const instance = document.querySelector('.CodeMirror').CodeMirror;
-      instance.setValue(content);
+      codemirror().setValue(content);
       await editPage.save();
 
       assert.strictEqual(
@@ -349,9 +346,8 @@ module('Acceptance | secrets/secret/create, read, delete', function (hooks) {
         'redirects to the show page'
       );
       assert.ok(showPage.editIsPresent, 'shows the edit button');
-      const savedInstance = document.querySelector('.CodeMirror').CodeMirror;
       assert.strictEqual(
-        savedInstance.options.value,
+        codemirror().options.value,
         JSON.stringify({ bar: 'boo', foo: 'fa' }, null, 2),
         'saves the content'
       );
