@@ -3,7 +3,7 @@
  * SPDX-License-Identifier: BUSL-1.1
  */
 
-import { click, fillIn, find, currentURL, settled, visit, waitUntil, findAll } from '@ember/test-helpers';
+import { click, fillIn, find, currentURL, settled, visit, findAll } from '@ember/test-helpers';
 import { module, test } from 'qunit';
 import { setupApplicationTest } from 'ember-qunit';
 import { v4 as uuidv4 } from 'uuid';
@@ -11,19 +11,20 @@ import { v4 as uuidv4 } from 'uuid';
 import { encodeString } from 'vault/utils/b64';
 import authPage from 'vault/tests/pages/auth';
 import { deleteEngineCmd, mountEngineCmd, runCmd } from 'vault/tests/helpers/commands';
+import codemirror from 'vault/tests/helpers/codemirror';
+import { GENERAL } from '../helpers/general-selectors';
 
 const SELECTORS = {
   secretLink: '[data-test-secret-link]',
   popupMenu: '[data-test-popup-menu-trigger]',
   versionsTab: '[data-test-transit-link="versions"]',
   actionsTab: '[data-test-transit-key-actions-link]',
-  rootCrumb: '[data-test-secret-root-link]',
   card: (action) => `[data-test-transit-card="${action}"]`,
   infoRow: (label) => `[data-test-value-div="${label}"]`,
   form: (item) => `[data-test-transit-key="${item}"]`,
   versionRow: (version) => `[data-test-transit-version="${version}"]`,
   rotate: {
-    trigger: '[data-test-confirm-action-trigger]',
+    trigger: '[data-test-transit-key-rotate]',
     confirm: '[data-test-confirm-button]',
   },
 };
@@ -37,14 +38,14 @@ const testConvergentEncryption = async function (assert, keyName) {
       encodePlaintext: false,
       encodeContext: false,
       assertAfterEncrypt: (key) => {
-        assert.dom('.modal.is-active').exists(`${key}: Modal opens after encrypt`);
+        assert.dom('[data-test-encrypt-modal]').exists(`${key}: Modal opens after encrypt`);
         assert.ok(
           /vault:/.test(find('[data-test-encrypted-value="ciphertext"]').innerText),
           `${key}: ciphertext shows a vault-prefixed ciphertext`
         );
       },
       assertBeforeDecrypt: (key) => {
-        assert.dom('.modal.is-active').doesNotExist(`${key}: Modal not open before decrypt`);
+        assert.dom('[data-test-decrypt-modal]').doesNotExist(`${key}: Modal not open before decrypt`);
         assert
           .dom('[data-test-transit-input="context"]')
           .hasValue(
@@ -52,9 +53,8 @@ const testConvergentEncryption = async function (assert, keyName) {
             `${key}: the ui shows the base64-encoded context`
           );
       },
-
       assertAfterDecrypt: (key) => {
-        assert.dom('.modal.is-active').exists(`${key}: Modal opens after decrypt`);
+        assert.dom('[data-test-decrypt-modal]').exists(`${key}: Modal opens after decrypt`);
         assert.strictEqual(
           find('[data-test-encrypted-value="plaintext"]').innerText,
           'NaXud2QW7KjyK6Me9ggh+zmnCeBGdG93LQED49PtoOI=',
@@ -69,20 +69,20 @@ const testConvergentEncryption = async function (assert, keyName) {
       encodePlaintext: false,
       encodeContext: false,
       assertAfterEncrypt: (key) => {
-        assert.dom('.modal.is-active').exists(`${key}: Modal opens after encrypt`);
+        assert.dom('[data-test-encrypt-modal]').exists(`${key}: Modal opens after encrypt`);
         assert.ok(
           /vault:/.test(find('[data-test-encrypted-value="ciphertext"]').innerText),
           `${key}: ciphertext shows a vault-prefixed ciphertext`
         );
       },
       assertBeforeDecrypt: (key) => {
-        assert.dom('.modal.is-active').doesNotExist(`${key}: Modal not open before decrypt`);
+        assert.dom('[data-test-decrypt-modal]').doesNotExist(`${key}: Modal not open before decrypt`);
         assert
           .dom('[data-test-transit-input="context"]')
           .hasValue(encodeString('context'), `${key}: the ui shows the input context`);
       },
       assertAfterDecrypt: (key) => {
-        assert.dom('.modal.is-active').exists(`${key}: Modal opens after decrypt`);
+        assert.dom('[data-test-decrypt-modal]').exists(`${key}: Modal opens after decrypt`);
         assert.strictEqual(
           find('[data-test-encrypted-value="plaintext"]').innerText,
           'NaXud2QW7KjyK6Me9ggh+zmnCeBGdG93LQED49PtoOI=',
@@ -97,20 +97,20 @@ const testConvergentEncryption = async function (assert, keyName) {
       encodePlaintext: false,
       encodeContext: false,
       assertAfterEncrypt: (key) => {
-        assert.dom('.modal.is-active').exists(`${key}: Modal opens after encrypt`);
+        assert.dom('[data-test-encrypt-modal]').exists(`${key}: Modal opens after encrypt`);
         assert.ok(
           /vault:/.test(find('[data-test-encrypted-value="ciphertext"]').innerText),
           `${key}: ciphertext shows a vault-prefixed ciphertext`
         );
       },
       assertBeforeDecrypt: (key) => {
-        assert.dom('.modal.is-active').doesNotExist(`${key}: Modal not open before decrypt`);
+        assert.dom('[data-test-decrypt-modal]').doesNotExist(`${key}: Modal not open before decrypt`);
         assert
           .dom('[data-test-transit-input="context"]')
           .hasValue(encodeString('context'), `${key}: the ui shows the input context`);
       },
       assertAfterDecrypt: (key) => {
-        assert.dom('.modal.is-active').exists(`${key}: Modal opens after decrypt`);
+        assert.dom('[data-test-decrypt-modal]').exists(`${key}: Modal opens after decrypt`);
         assert.strictEqual(
           find('[data-test-encrypted-value="plaintext"]').innerText,
           encodeString('This is the secret'),
@@ -125,20 +125,20 @@ const testConvergentEncryption = async function (assert, keyName) {
       encodePlaintext: true,
       encodeContext: true,
       assertAfterEncrypt: (key) => {
-        assert.dom('.modal.is-active').exists(`${key}: Modal opens after encrypt`);
+        assert.dom('[data-test-encrypt-modal]').exists(`${key}: Modal opens after encrypt`);
         assert.ok(
           /vault:/.test(find('[data-test-encrypted-value="ciphertext"]').innerText),
           `${key}: ciphertext shows a vault-prefixed ciphertext`
         );
       },
       assertBeforeDecrypt: (key) => {
-        assert.dom('.modal.is-active').doesNotExist(`${key}: Modal not open before decrypt`);
+        assert.dom('[data-test-decrypt-modal]').doesNotExist(`${key}: Modal not open before decrypt`);
         assert
           .dom('[data-test-transit-input="context"]')
           .hasValue(encodeString('secret 2'), `${key}: the ui shows the encoded context`);
       },
       assertAfterDecrypt: (key) => {
-        assert.dom('.modal.is-active').exists(`${key}: Modal opens after decrypt`);
+        assert.dom('[data-test-decrypt-modal]').exists(`${key}: Modal opens after decrypt`);
         assert.strictEqual(
           find('[data-test-encrypted-value="plaintext"]').innerText,
           encodeString('There are many secrets 🤐'),
@@ -151,7 +151,7 @@ const testConvergentEncryption = async function (assert, keyName) {
   for (const testCase of tests) {
     await click('[data-test-transit-action-link="encrypt"]');
 
-    find('#plaintext-control .CodeMirror').CodeMirror.setValue(testCase.plaintext);
+    codemirror('#plaintext-control').setValue(testCase.plaintext);
     await fillIn('[data-test-transit-input="context"]', testCase.context);
 
     if (!testCase.encodePlaintext) {
@@ -161,7 +161,7 @@ const testConvergentEncryption = async function (assert, keyName) {
     if (testCase.encodeContext) {
       await click('[data-test-transit-b64-toggle="context"]');
     }
-    assert.dom('.modal.is-active').doesNotExist(`${name}: is not open before encrypt`);
+    assert.dom('[data-test-encrypt-modal]').doesNotExist(`${keyName}: is not open before encrypt`);
     await click('[data-test-button-encrypt]');
 
     if (testCase.assertAfterEncrypt) {
@@ -170,16 +170,17 @@ const testConvergentEncryption = async function (assert, keyName) {
     }
     // store ciphertext for decryption step
     const copiedCiphertext = find('[data-test-encrypted-value="ciphertext"]').innerText;
-    await click('.modal.is-active [data-test-modal-background]');
+    await click('dialog button');
 
-    assert.dom('.modal.is-active').doesNotExist(`${name}: Modal closes after background clicked`);
+    assert.dom('dialog.hds-modal').doesNotExist(`${keyName}: Modal closes after background clicked`);
     await click('[data-test-transit-action-link="decrypt"]');
 
     if (testCase.assertBeforeDecrypt) {
       await settled();
       testCase.assertBeforeDecrypt(keyName);
     }
-    find('#ciphertext-control .CodeMirror').CodeMirror.setValue(copiedCiphertext);
+
+    codemirror('#ciphertext-control').setValue(copiedCiphertext);
     await click('[data-test-button-decrypt]');
 
     if (testCase.assertAfterDecrypt) {
@@ -187,9 +188,9 @@ const testConvergentEncryption = async function (assert, keyName) {
       testCase.assertAfterDecrypt(keyName);
     }
 
-    await click('.modal.is-active [data-test-modal-background]');
+    await click('dialog button');
 
-    assert.dom('.modal.is-active').doesNotExist(`${name}: Modal closes after background clicked`);
+    assert.dom('dialog.hds-modal').doesNotExist(`${keyName}: Modal closes after background clicked`);
   }
 };
 
@@ -238,14 +239,18 @@ module('Acceptance | transit (flaky)', function (hooks) {
     await click('[data-test-toggle-label="Auto-rotation period"]');
     await click(SELECTORS.form('create'));
 
-    assert.strictEqual(currentURL(), `/vault/secrets/${this.path}/show/${name}`, 'it navigates to show page');
+    assert.strictEqual(
+      currentURL(),
+      `/vault/secrets/${this.path}/show/${name}?tab=details`,
+      'it navigates to show page'
+    );
     assert.dom(SELECTORS.infoRow('Auto-rotation period')).hasText('30 days');
     assert.dom(SELECTORS.infoRow('Deletion allowed')).hasText('false');
     assert.dom(SELECTORS.infoRow('Derived')).hasText('Yes');
     assert.dom(SELECTORS.infoRow('Convergent encryption')).hasText('Yes');
-    await click(SELECTORS.rootCrumb);
+    await click(GENERAL.breadcrumbLink(this.path));
     await click(SELECTORS.popupMenu);
-    const actions = findAll('.ember-basic-dropdown-content li');
+    const actions = findAll('.hds-dropdown__list li');
     assert.strictEqual(actions.length, 2, 'shows 2 items in popup menu');
 
     await click(SELECTORS.secretLink);
@@ -345,17 +350,15 @@ module('Acceptance | transit (flaky)', function (hooks) {
 
     await click(SELECTORS.versionsTab);
     assert.dom(SELECTORS.versionRow(1)).hasTextContaining('Version 1', `${name}: only one key version`);
-    await waitUntil(() => find(SELECTORS.rotate.trigger));
+
     await click(SELECTORS.rotate.trigger);
     await click(SELECTORS.rotate.confirm);
-    // wait for rotate call
-    await waitUntil(() => find(SELECTORS.versionRow(2)));
+
     assert.dom(SELECTORS.versionRow(2)).exists('two key versions after rotate');
 
     // navigate back to actions tab
     await click(SELECTORS.actionsTab);
 
-    await waitUntil(() => find(SELECTORS.card('encrypt')));
     assert.dom(SELECTORS.card('encrypt')).exists(`renders encrypt action card for ${name}`);
     await click(SELECTORS.card('encrypt'));
     assert
@@ -367,11 +370,6 @@ module('Acceptance | transit (flaky)', function (hooks) {
     await testConvergentEncryption(assert, name);
   });
 
-  /* 
-  OLD FLAKY TESTS (skipped)
-  It's been a while since we've updated the transit engine
-  keeping these tests to run locally the next time we touch that secret engine
-  */
   const KEY_TYPE_COMBINATIONS = [
     {
       name: (uid) => `aes-${uid}`,
@@ -459,7 +457,7 @@ module('Acceptance | transit (flaky)', function (hooks) {
   ];
 
   for (const key of KEY_TYPE_COMBINATIONS) {
-    test.skip(`transit backend: ${key.type}`, async function (assert) {
+    test(`transit backend: ${key.type}`, async function (assert) {
       assert.expect(key.convergent ? 43 : 7);
       const name = await this.generateTransitKey(key);
       await visit(`vault/secrets/${this.path}/show/${name}`);
@@ -473,12 +471,9 @@ module('Acceptance | transit (flaky)', function (hooks) {
       // wait for capabilities
 
       assert.dom('[data-test-transit-version]').exists({ count: 1 }, `${name}: only one key version`);
-      await waitUntil(() => find(SELECTORS.rotate.trigger));
       await click(SELECTORS.rotate.trigger);
 
       await click(SELECTORS.rotate.confirm);
-      // wait for rotate call
-      await waitUntil(() => findAll('[data-test-transit-version]').length >= 2);
       assert
         .dom('[data-test-transit-version]')
         .exists({ count: 2 }, `${name}: two key versions after rotate`);
@@ -491,7 +486,6 @@ module('Acceptance | transit (flaky)', function (hooks) {
       );
 
       const keyAction = key.supportsEncryption ? 'encrypt' : 'sign';
-      await waitUntil(() => find(`[data-test-transit-action-title=${keyAction}]`));
 
       assert
         .dom(`[data-test-transit-action-title=${keyAction}]`)
