@@ -13,16 +13,8 @@ module('Integration | Component | transform-role-edit', function (hooks) {
 
   skip('it renders', async function (assert) {
     // TODO: Fill out these tests, merging without to unblock other work
-
-    await render(hbs`{{transform-role-edit}}`);
-
-    assert.dom(this.element).hasText('');
-
-    // Template block usage:
     await render(hbs`
-      {{#transform-role-edit}}
-        template block text
-      {{/transform-role-edit}}
+      <TransformRoleEdit />
     `);
 
     assert.dom(this.element).hasText('template block text');
