@@ -12,7 +12,7 @@ module('Integration | Helper | add-to-array', function (hooks) {
 
   test('it correctly adds a value to an array without mutating the original', function (assert) {
     const ARRAY = ['horse', 'cow', 'chicken'];
-    const result = addToArray([ARRAY, 'pig']);
+    const result = addToArray(ARRAY, 'pig');
     assert.deepEqual(result, [...ARRAY, 'pig'], 'Result has additional item');
     assert.deepEqual(ARRAY, ['horse', 'cow', 'chicken'], 'original array is not mutated');
   });
@@ -20,7 +20,7 @@ module('Integration | Helper | add-to-array', function (hooks) {
   test('it fails if the first value is not an array', function (assert) {
     let result;
     try {
-      result = addToArray(['not-array', 'string']);
+      result = addToArray('not-array', 'string');
     } catch (e) {
       result = e.message;
     }
@@ -29,13 +29,13 @@ module('Integration | Helper | add-to-array', function (hooks) {
 
   test('it works with non-string arrays', function (assert) {
     const ARRAY = ['five', 6, '7'];
-    const result = addToArray([ARRAY, 10]);
+    const result = addToArray(ARRAY, 10);
     assert.deepEqual(result, ['five', 6, '7', 10], 'added number value');
   });
 
   test('it de-dupes the result', function (assert) {
     const ARRAY = ['horse', 'cow', 'chicken'];
-    const result = addToArray([ARRAY, 'horse']);
+    const result = addToArray(ARRAY, 'horse');
     assert.deepEqual(result, ['horse', 'cow', 'chicken']);
   });
 });
