@@ -16,11 +16,7 @@ export default create({
   configLink: clickable('[data-test-engine-config]', {
     testContainer: '#ember-testing',
   }),
-  disableButton: clickable('[data-test-confirm-action-trigger]', {
-    testContainer: '#ember-testing',
-  }),
-  confirmDisable: clickable('[data-test-confirm-button]', {
-    testContainer: '#ember-testing',
-  }),
+  disableButton: clickable('[data-test-confirm-action-trigger]'),
+  confirmDisable: clickable('[data-test-confirm-button]'),
   console: uiPanel,
 });
