@@ -51,8 +51,8 @@ module('Unit | Service | control group', function (hooks) {
 
   hooks.afterEach(function () {});
 
-  const isOSS = (context) => set(context, 'version.isOSS', true);
-  const isEnt = (context) => set(context, 'version.isOSS', false);
+  const isCommunity = (context) => set(context, 'version.type', 'community');
+  const isEnt = (context) => set(context, 'version.type', 'enterprise');
   const resolvesArgs = (assert, result, expectedArgs) => {
     return result.then((...args) => {
       return assert.deepEqual(args, expectedArgs, 'resolves with the passed args');
@@ -61,31 +61,31 @@ module('Unit | Service | control group', function (hooks) {
 
   [
     [
-      'it resolves isOSS:true, wrapTTL: true, response: has wrap_info',
-      isOSS,
+      'it resolves isCommunity:true, wrapTTL: true, response: has wrap_info',
+      isCommunity,
       [[{ one: 'two', three: 'four' }], { wrap_info: { token: 'foo', accessor: 'bar' } }, true],
       (assert, result) => resolvesArgs(assert, result, [{ one: 'two', three: 'four' }]),
     ],
     [
-      'it resolves isOSS:true, wrapTTL: false, response: has no wrap_info',
-      isOSS,
+      'it resolves isCommunity:true, wrapTTL: false, response: has no wrap_info',
+      isCommunity,
       [[{ one: 'two', three: 'four' }], { wrap_info: null }, false],
       (assert, result) => resolvesArgs(assert, result, [{ one: 'two', three: 'four' }]),
     ],
     [
-      'it resolves isOSS: false and wrapTTL:true response: has wrap_info',
+      'it resolves isCommunity: false and wrapTTL:true response: has wrap_info',
       isEnt,
       [[{ one: 'two', three: 'four' }], { wrap_info: { token: 'foo', accessor: 'bar' } }, true],
       (assert, result) => resolvesArgs(assert, result, [{ one: 'two', three: 'four' }]),
     ],
     [
-      'it resolves isOSS: false and wrapTTL:false response: has no wrap_info',
+      'it resolves isCommunity: false and wrapTTL:false response: has no wrap_info',
       isEnt,
       [[{ one: 'two', three: 'four' }], { wrap_info: null }, false],
       (assert, result) => resolvesArgs(assert, result, [{ one: 'two', three: 'four' }]),
     ],
     [
-      'it rejects isOSS: false, wrapTTL:false, response: has wrap_info',
+      'it rejects isCommunity: false, wrapTTL:false, response: has wrap_info',
       isEnt,
       [
         [{ one: 'two', three: 'four' }],
@@ -107,7 +107,8 @@ module('Unit | Service | control group', function (hooks) {
     ],
   ].forEach(function ([name, setup, args, expectation]) {
     test(`checkForControlGroup: ${name}`, function (assert) {
-      const assertCount = name === 'it rejects isOSS: false, wrapTTL:false, response: has wrap_info' ? 2 : 1;
+      const assertCount =
+        name === 'it rejects isCommunity: false, wrapTTL:false, response: has wrap_info' ? 2 : 1;
       assert.expect(assertCount);
       if (setup) {
         setup(this);
