@@ -7,6 +7,8 @@ import (
 	"io"
 	"net"
 	"time"
+
+	uberAtomic "go.uber.org/atomic"
 )
 
 type delayedConn struct {
@@ -15,12 +17,13 @@ type delayedConn struct {
 }
 
 func newDelayedConn(conn net.Conn, delay time.Duration) net.Conn {
+	dr := &delayedReader{
+		r:     conn,
+		delay: uberAtomic.NewDuration(delay),
+	}
 	return &delayedConn{
+		dr:   dr,
 		Conn: conn,
-		dr: &delayedReader{
-			r:     conn,
-			delay: delay,
-		},
 	}
 }
 
@@ -29,18 +32,18 @@ func (conn *delayedConn) Read(data []byte) (int, error) {
 }
 
 func (conn *delayedConn) SetDelay(delay time.Duration) {
-	conn.dr.delay = delay
+	conn.dr.delay.Store(delay)
 }
 
 type delayedReader struct {
 	r     io.Reader
-	delay time.Duration
+	delay *uberAtomic.Duration
 }
 
 func (dr *delayedReader) Read(data []byte) (int, error) {
 	// Sleep for the delay period prior to reading
-	if dr.delay > 0 {
-		time.Sleep(dr.delay)
+	if delay := dr.delay.Load(); delay != 0 {
+		time.Sleep(delay)
 	}
 
 	return dr.r.Read(data)
