@@ -13,6 +13,7 @@ import (
 	"github.com/hashicorp/go-memdb"
 	"github.com/hashicorp/vault/helper/namespace"
 	"github.com/hashicorp/vault/sdk/framework"
+	"github.com/hashicorp/vault/sdk/helper/consts"
 	"github.com/hashicorp/vault/sdk/logical"
 )
 
@@ -38,6 +39,8 @@ var (
 		return nil
 	}
 
+	sysActivityLogReporting = func(b *SystemBackend) {}
+
 	getSystemSchemas = func() []func() *memdb.TableSchema { return nil }
 
 	getEGPListResponseKeyInfo = func(*SystemBackend, *namespace.Namespace) map[string]interface{} { return nil }
@@ -263,7 +266,7 @@ var (
 
 		return paths
 	}
-	handleGlobalPluginReload = func(context.Context, *Core, string, string, []string) error {
+	handleGlobalPluginReload = func(context.Context, *Core, pluginReloadRequest) error {
 		return nil
 	}
 	handleSetupPluginReload = func(*Core) error {
@@ -277,6 +280,16 @@ var (
 	checkRaw = func(b *SystemBackend, path string) error { return nil }
 )
 
+// Contains the config for a global plugin reload
+type pluginReloadRequest struct {
+	Type       string            `json:"type"` // Either 'plugins' or 'mounts'
+	PluginType consts.PluginType `json:"plugin_type"`
+	Subjects   []string          `json:"subjects"`  // The plugin names or mount points for the reload
+	ReloadID   string            `json:"reload_id"` // a UUID for the request
+	Timestamp  time.Time         `json:"timestamp"`
+	Namespace  *namespace.Namespace
+}
+
 // tuneMount is used to set config on a mount point
 func (b *SystemBackend) tuneMountTTLs(ctx context.Context, path string, me *MountEntry, newDefault, newMax time.Duration) error {
 	zero := time.Duration(0)
@@ -314,7 +327,7 @@ func (b *SystemBackend) tuneMountTTLs(ctx context.Context, path string, me *Moun
 	if err != nil {
 		me.Config.MaxLeaseTTL = origMax
 		me.Config.DefaultLeaseTTL = origDefault
-		return fmt.Errorf("failed to update mount table, rolling back TTL changes")
+		return fmt.Errorf("failed to update mount table, rolling back TTL changes: %w", err)
 	}
 	if b.Core.logger.IsInfo() {
 		b.Core.logger.Info("mount tuning of leases successful", "path", path)
